import os
from shutil import copy
import subprocess



def generate_pymol(xyz_directory):
    print("If you do not want to generate a pymol session, enter 'exit'")
    filenames_to_pymol = input("File Names: ").split()
    if filenames_to_pymol[0].lower() == "exit":
        return

    print("File Names Received")
    print("Files to print below")
    for file in filenames_to_pymol:
        print(file)

    print(filenames_to_pymol)
    print()
    print("Please create a New Folder to copy structures into")
    current_directory = os.getcwd()
    added_xyz_folder_path = input("Enter New Folder Name: ")
    xyz_folder_path = os.path.join(current_directory,added_xyz_folder_path)

    os.mkdir(xyz_folder_path)
    print("Folder Created")
    # user_input_directory = "/Users/matthewnwerem/Chapman University/OMO Research Group - Project Conf. Analysis - Nwerem - proj_conf_analysis/Code Versions/Conformational_Analysis/pentane_file_test"
    user_input_directory = xyz_directory
    xyz_files_dir = os.listdir(user_input_directory)

    xyz_files = []
    for file in xyz_files_dir:
        if file.endswith(".xyz"):
            xyz_files.append(file)

    print("Printing all possible XYZ Files in question to be able to see in PyMol ")
    print(xyz_files)

    #  if there is a file in filenames_to_Pymol, that matches a file in xyz_files_dir, then
    for current_file in filenames_to_pymol:
        for current_file2 in xyz_files:
            if current_file == current_file2:
                #  copy file to the new directory path
                file_name = current_file2
                print("Copying " + file_name + " ...")
                print(xyz_folder_path)
                current_file_path = os.path.join(user_input_directory, file_name)
                copy(current_file_path, xyz_folder_path)

    os.chdir(xyz_folder_path)
    print("Directory Changed")
    # subprocess.Popen("mGenerate_PyMOL_Session.sh", cwd=None, shell= True)
    subprocess.Popen("mGenerate_PyMOL_Session.sh", cwd=None, shell= True)


# testing function
#/Users/matthewnwerem/Chapman University/OMO Research Group - Project Conf. Analysis - Nwerem - proj_conf_analysis/Code Versions/Conformational_Analysis/pentane_file_test
# generate_pymol("/Users/matthewnwerem/Chapman University/OMO Research Group - Project Conf. Analysis - Nwerem - proj_conf_analysis/Code Versions/Conformational_Analysis/pentane_file_test")



#take in post clustering pd.df that contains name of file and its corresponding family

def family_files(df_pc_clustering_final, num_clusters,files_directory):
    directory_files = os.listdir(files_directory) #this is the directory that they typed in at the beginning
    xyz_files = []
    
    #change directory to location of xyz files
    os.chdir(files_directory)

    #create folder-->
    family_holder_name = input("Enter folder name for families: ")
    current_file_path3 = os.path.join(os.getcwd(),family_holder_name)
    os.mkdir(current_file_path3)
    os.chdir(current_file_path3)

    family_directories = []
    
    for nums in range(1,num_clusters+1):
        #create folder
        new_directory = "Family " + str(nums)
        path = os.path.join(current_file_path3,new_directory)
        os.mkdir(path)
        family_directories.append(path) #save for later ;)
        
    # at this point, all folders have been created
    
    for index, row in df_pc_clustering_final.iterrows():
        family = row["Families"]
        current_file = os.path.join(files_directory,index)
        new_file_location = os.path.join(current_file_path3,family)
        copy(current_file,new_file_location)
        
    print()
    print("Files have been put in their respective family folders")
    
    
    
    # 
    # for file in directory_files:
    #     if file.endswith(".xyz"):
    #         #at this point, we are with a specific file. ex. tamylOH_7.xyz
    #         
    #         for row in df:
    #             if row.index (name) == file:
    #                 copy file to specific family directory
    #                 family = row["Families"]
    #                 path2 = os.path.join(directory_files,family)
    # 
    #                 from shutil import copyfile
    #                 copyfile(file (has to be full path), dst)
    #                 row["Families"]
                    









import os
import xyz_old_version
import xyz
import pca_creation
import pymol
import clustering
import stat
from datetime import datetime

startTime = datetime.now()
print(startTime)
print("What is the directory in which your xyz files are in?")

class PathNoExistError(Exception):
    def __init__(self, message):
        self.message = message


class NoXYZFileExistError(Exception):
    def __init__(self, message):
        self.message = message


class InvalidPCAError(Exception):
    def __init__(self, message):
        self.message = message


class InvalidClusterError(Exception):
    def __init__(self, message):
        self.message = message


class InvalidUserEntranceError(Exception):
    def __init__(self, message):
        self.message = message


attempts_left = 10
for attempt in range(10):  # user has ten times to get a correct path, otherwise must restart
    try:
        user_input_directory = input("XYZ File Directory Path: ")
        # user_input_directory = "/Users/matthewnwerem/Chapman University/OMO Research Group - Project Conf. Analysis - Nwerem - proj_conf_analysis/Code Versions/Conformational_Analysis/cycloheptadecane_file_test2"
        # user_input_directory = "/Users/matthewnwerem/Chapman University/OMO Research Group - Project Conf. Analysis - Nwerem - proj_conf_analysis/Code Versions/Conformational_Analysis/cycloheptadecane_file_test"
        # user_input_directory = "/Users/matthewnwerem/Chapman University/OMO Research Group - Project Conf. Analysis - Nwerem - proj_conf_analysis/Code Versions/Conformational_Analysis/cyclooctane_file_test"
        # user_input_directory = "/Users/matthewnwerem/Chapman University/OMO Research Group - Project Conf. Analysis - Nwerem - proj_conf_analysis/Code Versions/Conformational_Analysis/pentane_file_test"
        # user_input_directory = "/Users/matthewnwerem/Chapman University/OMO Research Group - Project Conf. Analysis - Nwerem - proj_conf_analysis/Code Versions/Conformational_Analysis/xyz_file_test"
        # user_input_directory = "/Users/matthewnwerem/Chapman University/OMO Research Group - Project Conf. Analysis - Nwerem - proj_conf_analysis/Code Versions/Conformational_Analysis/1_4_dioxepane_file_test"
        # user_input_directory = "/Users/matthewnwerem/Chapman University/OMO Research Group - Project Conf. Analysis - Nwerem - proj_conf_analysis/Code Versions/Conformational_Analysis/1fluoro4propylcyclohexane"
        user_input_directory = "/Users/matthewnwerem/Chapman University/OMO Research Group - Project Conf. Analysis - Nwerem - proj_conf_analysis/Code Versions/Conformational_Analysis/test_files/cis_1fluoro4propylcyclohexane"

        # 1,4,7,9,12,14,18,21,24,27,30,33,36,39,42,45,46
        while not os.path.exists(user_input_directory):
            raise PathNoExistError("Path: '{}' does not exist".format(user_input_directory))

        print("______________________________________________________")
        print(user_input_directory)
        print("Path Received")
        print("______________________________________________________")

        directory_files = os.listdir(user_input_directory)
        xyz_files = []

        for file in directory_files:
            if file.endswith(".xyz"):
                xyz_files.append(file)

        if not xyz_files:  # means if the list is empty (using implicit booleanness of empty lists)
            raise NoXYZFileExistError("There are no XYZ files in the path '{}".format(user_input_directory))

        # print("Current Test XYZ File:", xyz_files[0]) #0 because first index of a LIST
        # print("Type w/ [0]", type(xyz_files[0]))
        # print("Type w/o [0]", type(xyz_files))

        # print(xyz_files)
        # complete_path = user_input_directory+"/"+xyz_files[0] #need complete path to open file
        # specific_file_object = open(complete_path)  # will need to make loop for each xyzfile when doing multiple i think

        xyz_df = xyz.xyz_contents(xyz_files, user_input_directory)
        print("______________________________________________________")
        print("Current Dataframe")
        print("Has been tested and works with")
        print("1: Adding all xyz's of t-amlyOH1.xyz from a folder to a dataframe")
        print("2: Adding a second xyz (just another copy of amylOH_1) to the same dataframe ")
        print("3: Calculating distances angles and dihedrals")
        print("4: Adding distances, angles and dihedrals to dataframe")
        print("______________________________________________________")
        print(xyz_df)
        print(datetime.now() - startTime)


    except PathNoExistError as e:
        print()
        print(e.message)
        print()
        print("______________________________________________________")
        attempts_left -= 1
        print(attempts_left, "attempts left")
        continue
    except NoXYZFileExistError as e:
        print()
        print(e.message)
        print()
        print("______________________________________________________")
        attempts_left -= 1
        print(attempts_left, "attempts left")
        continue

    break

attempts_left = 10
print("______________________________________________________")
print("PCA")
print("Has been tested and works with")
print("1: Creating PCA Model and its subsequent plots")
print("2: all interactive plots show correct hover name ")
print("3: showing top 10 features that effect the PC")
print("______________________________________________________")

for attempt in range(10):  # user has ten times to get a correct num of pcs desired, otherwise must restart
    try:
        testPCs = True
        while testPCs:
            userPCA_Components = input("Enter the number of components you would like to see in the model: ")
            userPCA_Components = int(userPCA_Components)

            while userPCA_Components >= xyz_df.shape[0]:
                raise InvalidPCAError("The # of components must be less than {}".format(xyz_df.shape[0]))

            userPCA_model, user_num_components,original_df = pca_creation.pca_model(userPCA_Components, xyz_df)

            print()
            print("Would you like to test with a different number of PCs?")
            testing_PCs = input("Enter Y/N: ").upper()
            print()

            if testing_PCs == "Y":
                testPCs = True

            elif testing_PCs == "N":
                testPCs = False
                print("Ending...")

            while testing_PCs != "Y" and testing_PCs != "N":
                raise InvalidUserEntranceError("You did not choose Y or N, try again.")

    except InvalidPCAError as e:
        print()
        print(e.message)
        print("Your number of components desired and dataframe dimensions do not work well for PCA")
        print()
        print("______________________________________________________")
        attempts_left -= 1
        print(attempts_left, "attempts left")
        continue

    except ValueError:
        print()
        print("Not an integer")
        attempts_left -= 1
        print(attempts_left, "attempts left")
        continue

    except InvalidUserEntranceError as e:
        print()
        print(e.message)
        print()
        print("______________________________________________________")
        attempts_left -= 1
        print(attempts_left, "attempts left")
        continue

    break

attempts_left = 10
print("______________________________________________________")
print("Clustering")
print("Has been tested and works with")
print("1: Creating Clustering Model with PCA data and its subsequent plots")
print("2: All interactive plots show correct hover name ")
print("______________________________________________________")

for attempt in range(10):  # user has ten times to get a correct num of pcs desired, otherwise must restart
    try:
        testClusters = True
        while testClusters:
            tested_kclusters = 10

            # see if you can do it in a way where it depends on the number of conformations brought in
            # aka like change tested clusters to be half of the shape of all. That way its better
            # userPCA_model.shape[0]/2 ?? or userPCA_model.shape[0]/5?? test out and see whats best

            while tested_kclusters >= userPCA_model.shape[0]:
                raise InvalidClusterError("The # of clusters to be tested is greater than {}".format(userPCA_model.shape[0]))
                #  if this is raised, it means that you are looking into less than 10 structures
                #  in that case, it will be tougher to find differentiation anyway

            userClustermodel = clustering.clustering_model(userPCA_model, user_num_components,original_df)

            print()
            print("Would you like to test with a different number of Clusters?")
            testing_Clusters = input("Enter Y/N: ").upper()
            print()

            if testing_Clusters == "Y":
                testClusters = True

            elif testing_Clusters == "N":
                testClusters = False
                print("Ending...")

            while testing_Clusters != "Y" and testing_Clusters != "N":
                raise InvalidUserEntranceError("You did not choose Y or N, try again.")

    except InvalidClusterError as e:
        print()
        print(e.message)
        print("Your number of components desired and dataframe dimensions do not work well for PCA")
        print()
        print("______________________________________________________")
        attempts_left -= 1
        print(attempts_left, "attempts left")
        continue

    # except ValueError:
    #     print()
    #     print("Not an integer")
    #     attempts_left -= 1
    #     print(attempts_left, "attempts left")
    #     continue

    except InvalidUserEntranceError as e:
        print()
        print(e.message)
        print()
        print("______________________________________________________")
        attempts_left -= 1
        print(attempts_left, "attempts left")
        continue

    break

attempts_left = 10
os.chmod('mGenerate_PyMOL_Session.sh', stat.S_IRWXU)
print("______________________________________________________")
print("Importing structures to PyMol")
print("After reviewing the given plots, which of the files would you like to view in pyMol?")
print("Enter the filename with  (ex. 'tamylOH_17.xyz'), with commas in between each name")
print("______________________________________________________")

for attempt in range(10):  # user has ten times to get a correct num of pcs desired, otherwise must restart
    try:
        pymolSession = True
        current_directory_overall = os.getcwd()

        while pymolSession:

            userPymolSession = pymol.generate_pymol(user_input_directory)
            print()
            print("Would you like to generate a PyMol Session with different structures in the current directory?")
            new_PyMolSession = input("Enter Y/N: ").upper()
            print()

            if new_PyMolSession == "Y":
                os.chdir(current_directory_overall)
                pymolSession = True

            elif new_PyMolSession == "N":
                pymolSession = False
                print("Ending...")

            while new_PyMolSession != "Y" and new_PyMolSession != "N":
                raise InvalidUserEntranceError("You did not choose Y or N, try again.")

    except InvalidUserEntranceError as e:
        print()
        print(e.message)
        print()
        print("______________________________________________________")
        attempts_left -= 1
        print(attempts_left, "attempts left")
        continue

    break









import matplotlib.pyplot as plt
from sklearn.cluster import KMeans
import plotly as py
import plotly.graph_objs as go
import plotly.express as px
# import plotly.tools as tls
import numpy as np
import pandas as pd
from mpl_toolkits.mplot3d import Axes3D #needed for 3D visualizations

import pca_creation
# column_headers = ["1 C to 2 H", "1 C to 3 H", "1 C to 4 C", "17 H to 16 H to 15 H to 12 H",
#                "17 H to 16 H to 15 H to 13 H", "17 H to 16 H to 15 H to 14 C"]
#
# indexes = ["pentane_1.xyz", "pentane_3.xyz", "pentane_2.xyz", "pentane_4_fake.xyz", "pentane_5_fake.xyz"]
#
#
# A = np.array([[1.096276,1.096276,1.528389,55.236766,71.552750,37.123885],
#               [1.096818,1.095462,1.530606,121.597386,100.494262,37.106736],
#               [1.097144,1.095743,1.531324,67.610414,128.966981,37.456963],
#               [1.530606,1.095462,1.196818,121.106736,100.966981,37.530606],
#               [128.966981,1.095743,1.531324,37.456963,1.097144,67.610414]])
#
#
# test_df = pd.DataFrame(A, columns=column_headers)
# test_df.index = indexes
#
# print(test_df.iloc[:,:2])
# print(test_df.index[0][:-6])

def clustering_model(pca_df,num_PCA_components,original_df):

    print("Clustering Analysis on PCA data commencing...")
    print("Clustering will be done on 1-10 clusters")

    kclusters = range(1,11)

    inertias = []
    #  we keep a list of all intertias for each model of clustering to see which one is the most accurate

    for k in kclusters:
        # create model and fit to data
        model = KMeans(n_clusters=k)
        model.fit(pca_df)
        inertias.append(model.inertia_)
        # inertia_ (kmeans attribute): Sum of squared distances of samples to their closest cluster center.

    # Method 1 for plotting inertias
    # plt.plot(kclusters, inertias, '-o', color='black')
    # plt.xlabel('number of clusters, k')
    # plt.ylabel('inertia')
    # plt.xticks(kclusters)
    # plt.show()
    # print(inertias)
    # print(kclusters)

    ten_clusters = []
    for i in range(1,11):
        ten_clusters.append(i)

    # print("CLUSTERRR:",ten_clusters)
    # print(inertias)
    # df_inertias = pd.DataFrame([inertias,ten_clusters]).transpose()
    # df_inertias.columns = ["Inertia","Number of Clusters"]
    # df_inertias.index += 1
    # print(df_inertias)

    # method 2 for showing inertia plot
    # fig1 = px.line(df_inertias, x="Number of Clusters",y="Inertia")
    # fig1.show()

    figtest = go.Figure()
    figtest.add_trace(go.Scatter(x=ten_clusters,y=inertias))
    figtest.update_layout(title='Inertias for choosing best number of clusters',
                      xaxis_title='Number of Clusters',
                      yaxis_title='Inertia')

    figtest.show()
    # inertias_np = np.array(inertias)
    # data1 = pgo.Scatter(x=kclusters,
    #         y=inertias_np)
    # layout1 = pgo.Layout(title="Inertias for choosing best number of clusters",
    #                      xaxis="Number of clusters",yaxis="Inertia")
    # fig1a = pgo.Figure(data1,layout1)
    # fig1a.show()


    #do the math to get the 'best' cluster
    # then tell em what it is but still allow user to choose # clusters


    print("After seeing the elbow point graph, how many clusters will you like to conduct for your data")
    user_clusters = int(input("Desired Clusters: "))
    print()
    print("Desired number of clusters received")

    user_Kmeans = KMeans(n_clusters= user_clusters)
    user_Kmeans.fit(pca_df)
    labels = user_Kmeans.predict(pca_df) # K means predicting the family

    families = []
    for integer in labels:
        integer+=1
        families.append("Family "+str(integer))   # creates the labeling of the graph look nicer, not just a number

    #Create dataframe containing pca data + families
    df_columns = []
    for num in range(1,num_PCA_components+1):
        df_columns.append("PC "+str(num))

    pca_df_final = pd.DataFrame(pca_df, columns= df_columns)

    pca_df_final["Families"] = families
    pca_df_final.index = original_df.index

    if num_PCA_components == 2:
        fig2 = px.scatter(pca_df_final, x="PC 1", y="PC 2",
                          title='2D Clustering',
                          hover_name=pca_df_final.index,
                          size_max=0.1,
                          color="Families")
        fig2.show()



    elif num_PCA_components == 3:
        fig3 = px.scatter_3d(pca_df_final, x="PC 1", y="PC 2", z="PC 3",
                             title='3D Clustering',
                             hover_name=pca_df_final.index,
                             color="Families",
                             size_max=0.1)
        fig3.show()

    else:
        print("You have asked for a not-optimal number of components for visualization ( >3 dimensions ) ")
        print("Each structure and their corresponding family has been printed in the terminal")

        #change to families.csv #### do this!!!
        # file_family_name = input("What would you like your csv file name?")
        file_family_name = pca_df_final.index[0][:-6] # will cut off _#.xyz from file name; might remove bc maybe not
                                                # everyone has that naming convention

        #take out just .xyz or something; split off of the "." and take everything to the left of it

        pca_df_final.to_csv(file_family_name+"_families.csv")

    print(pca_df_final)

    return pca_df_final,user_clusters


# def clustering_model(pca_df):
#
#     print("Clustering Analysis on PCA data commencing...")
#     print("Clustering will be done on 2-10 clusters")
#
#     kclusters = range(2,11)
#     RSSs = []
#     #we keep a list of all RSS's for each model of clustering to see which one is the most accurate
#
#     for k in kclusters:
#         # create model and fit to data
#         kmeans, z = cluster(i)
#         model = KMeans(n_clusters=k)
#         model.fit(pca_df)
#         RSSs.append(model.inertia_)
#         # inertia_ (kmeans attribute): Sum of squared distances of samples to their closest cluster center.
#
#     plt.plot(kclusters, RSSs, '-o', color='black')
#     plt.xlabel('number of clusters, k')
#     plt.ylabel('inertia')
#     plt.xticks(kclusters)
#     plt.show()
#
#     return plt.show()
#
# kclusters = range(2, 11)
# RSSs = []
# # we keep a list of all RSS's for each model of clustering to see which one is the most accurate
#
# for k in kclusters:
#     # create model and fit to data
#     model = KMeans(n_clusters=k)
#     model.fit(X)
#     RSSs.append(model.inertia_)
#     # inertia_ (kmeans attribute): Sum of squared distances of samples to their closest cluster center.
#
# plt.plot(kclusters, RSSs, '-o', color='black')
# plt.xlabel('number of clusters, k')
# plt.ylabel('inertia')
# plt.xticks(kclusters)
# plt.show()
#
# print("After seeing the elbow point graph, how many clusters will you like to conduct for your data")
# user_clusters = int(input("Desired Clusters: "))
#
# user_Kmeans = KMeans(n_clusters= user_clusters)
# user_Kmeans.fit(X)
# labels = user_Kmeans.predict(X)
#
# centroids = user_Kmeans.cluster_centers_
# u_labels = np.unique(labels)
#
# print(labels)
# print(centroids)
#
# plt.scatter(X[:,0], X[:,1], c =labels, cmap="viridis")
# plt.scatter(centroids[:,0], centroids[:,1], s=80, color='black', alpha=0.5)
# plt.legend()
# plt.show()




#
# atoms = 5
#
# atom_list = [1,2,3,4,5]
# # print(atom_list[:3])
#
# plane1 = []
# plane2 = []
#
# print()
# #take first three, add it to plane1, take last three, add it to plane two
# plane1.append(atom_list[:3])
# plane2.append(atom_list[1:4])
#
# plane1.append(atom_list[:3])
# plane2.append(atom_list[1:3] +[atom_list[4]])
#
# print("Current Planes")
# print(plane1)
# print(plane1[0])
# print()
# print(plane2)
# print(plane2[0])
#
#
# print("Planes to avoid")
# plane1_avoid = []
# plane2_avoid = []
#
# plane1_avoid.append(plane2[0][::-1]) #  reverse
# plane2_avoid.append(plane1[0][::-1]) #  reverse
#
# plane1_avoid.append(plane2[1][::-1]) #  reverse
# plane2_avoid.append(plane1[1][::-1]) #  reverse
#
# print(plane1_avoid)
# print(plane2_avoid)
#
# #[1 3 2 ] [3 2 5] same as [1 2 3] [2 3 5]
# print("tesdre")
#
# print(plane1)
# plane1_avoid.append((plane1[1][1],plane1[1][2] = plane1[1][2],plane1[1][1]))
# plane2_avoid.append((plane2[1][0],plane2[1][1] = plane2[1][1],plane2[1][0]))
# print(plane1)
# print(plane2)
# #plane2.append(atom_list[-4:2] +
# print()
#
# print(plane1_avoid)
# print(plane2_avoid)
#
# """
# 1 2 3 not 1 3 2
#                     second list
#                     2 3 1 can this be done? no; should not be done because 1 in first list
#                     2 3 2 no work should not be done because 1 in first list ++ more
#                     2 3 3 no work should not be done because 1 in first list ++ more
#                     2 3 4
#                     2 3 5
#                     (not 2 4 3 should not be done because 4 > 3 and first two indeces are not 2 3)
#                     (not 2 5 3 should not be done because 5 > 3 and first two indeces are not 2 3)
# 1 2 4
# 1 2 5
#
#
# 1 3 4
# 1 3 5
#
# 1 4 5
#
#
# 2 1 3 not 2 3 1
# 2 1 4
# 2 1 5
#
# 2 2 no work
#
# 2 3 1 should not be done because 3 > 1 (ie 2 1 3 has been done)
# 2 3 2 no work
# 2 3 4
# 2 3 5
#
# 2 4 1 should not be done because 4 > 1
# 2 4 2 no work
# 2 4 3 should not be done because 4 > 3
# 2 4 4 no work
# 2 4 5
#
# 3 1 2 not 3 2 1
# 3 1 3
# 3 1 4
# 3 1 5
#
# 3 2 1 should not be done because 2 > 1
# 3 2 2 no work
# 3 2 3 no work
# 3 2 4
# 3 2 5
#
# 3 3 no work
#
# 3 4 1 should not be done because 4 > 1
# 3 4 2 should not be done because 4 > 2
# 3 4 3 no work
# 3 4 4 no work
# 3 4 5
#                 second list
#                     4 5 1
#                     4 5 2
#                     4 5 3 no work should not be done because 1 in first list ++ more
#                     4 5 4 no
#                     4 5 5 no
#
#
#
# ...
#
# 5 1 2
# 5 1 3
# 5 1 4
# 5 1 5 no work
#
# 5 2 1 should not be done because 2 > 1 (ie 5 1 2)
# ...
#
# 5 4 3 should not be done because 4 > 3 (ie 3 4 5)
# """
#
# #Trying to think lol
# """
# input: number of atoms
# output: two lists of lists -->
#         list 1 = list of lists where each index contains the atom #/index to be in the plane
#         (ex. [5 1 2] is atom 5 atom 1 and atom 2
#
#         list 2 = list of lists where each index contains the atom #/index to be in the plane
#                 --> index matches with list 1 index
#
#         thus the first dihedrals calculated will be from
#         list1[0] and list2[0], then
#         list1[1] and list2[1] etc.
#
#         list1 = []
#         list2 = []
#         num_atoms = 5
#         atoms_list -> [1, 2, 3, 4, 5] #figure out how to generate
#         atoms_list = [1:num_atoms]
#         #first list
#         for atom_1 in atoms_list:
#
#             for atom_2 in atoms_list:
#
#                 if atom2 != atom1:
#
#                     for atom_3 in atoms_list:
#                         if atom3 > atom2 and atom3 != atom1:
#
#                             for atom_4 in num_atoms:
#                                 if atom_4 not in [atom_1, atom_2, atom_3]:
#                                     list1.append([atom1, atom2, atom3, atom4])
#                                     # list2.append([atom2,atom3,atom4])
#
#                     at the end list1 and list2 contain all the necessary pairs for dihedrals
#
#
#                     #Dihedrals
#                     for firstl, secondl in zip(list1, list2): #zip allows us to iterate through two lists simultaneously
#                         atom1 = tempxyzlist
#
#                         a1 = [float(temp_xyz_contents_list[3*firstl[0]-2]),
#                                   float(temp_xyz_contents_list[3*firstl[0]-1]),
#                                   float(temp_xyz_contents_list[3*firstl[0]])]
#                         a1_array = np.array(a1)
#
#                         a2 = [float(temp_xyz_contents_list[3*firstl[1]-2]),
#                                   float(temp_xyz_contents_list[3*firstl[1]-1]),
#                                   float(temp_xyz_contents_list[3*firstl[1]])]
#                         a2_array = np.array(a2)
#
#                         a3 = [float(temp_xyz_contents_list[3*firstl[2]-2]),
#                                   float(temp_xyz_contents_list[3*firstl[2]-1]),
#                                   float(temp_xyz_contents_list[3*firstl[2]])]
#                         a3_array = np.array(a3)
#
#                         a4 = [float(temp_xyz_contents_list[3*firstl[3]-2]),
#                                   float(temp_xyz_contents_list[3*firstl[3]-1]),
#                                   float(temp_xyz_contents_list[3*firstl[3]-2])]
#                         a4_array = np.array(a4)
#
#                         v1 = a2_array-a1_array
#                         v2 = a3_array - a2_array
#                         n1 = np.cross(v1,v2) #  cross product
#                         v3 = a2_array - a3_array
#                         v4 = a4_array - a3_array
#                         n2 = np.cross(v3,v4)
#
#                         dihedral = np.arccos(np.round(-np.dot(n1, n2) /
#                                           (np.linalg.norm(n1) *
#                                            np.linalg.norm(n2)), decimals=4)) #rads
#
#                         if np.dot(n1,v4) > 0:
#                             dihedral = -1*dihedral
#
#                         dihedral_degrees = np.degrees(dihedral)
#
#                         #MODIFICATION OF DIHEDRALS (NO LONGER ACTUAL DIHEDRALS)
#                         if dihedral_degrees > 90 and dihedral_degrees < 180:
#                             dihedral_degrees = 180 - dihedral_degrees
#
#                         elif dihedral_degrees < -90 and dihedral_degrees > -180:
#                             dihedral_degrees = -180-dihedral_degrees
#
#
#                         temp_dihedral_list.append(dihedral_degrees)
#
#                         dihedral_colname_list.append(temp_xyz_colname_list[3*firstl[0]][:-2] +
#                                                                                  "to " + temp_xyz_colname_list[3*firstl[1]][:-2] +
#                                                                                  "to " + temp_xyz_colname_list[3*firstl[2]][:-2] +
#                                                                                  "to " + temp_xyz_colname_list[3*firstl[3]][:-2])
#                                                     # Ex. 1 C to 12 H to 20 F to 21 C
#
#                     master_dihedral_contents_list.append(temp_dihedral_list)
#
#
#
#
# """
#
#
#
#
#
#
# # for combinations in atom_list:

test = [[1, 2 ,3],[3,2,1]]
print(test)

for list in reversed(test):
    if list[::-1] in test:
        print(list)
        test.remove(list)


print(test)




# Create dataframe with xyz contents
import pandas as pd
import numpy as np
import sys
import math
master_xyz_contents_list = []
master_dist_contents_list = []
master_angle_contents_list = []
master_dihedral_contents_list = []
calculations_list = []


class InvalidUserEntranceError(Exception):
    def __init__(self, message):
        self.message = message


#  Goal 1: Grab XYZ Content


def xyz_contents(xyz_files, user_directory):
    xyz_index_list = []
    comp_distances = False
    comp_angles = False
    comp_dihedrals = False

    while True:
        try:
            complete_path = user_directory + "/" + xyz_files[0]
            specific_file_object = open(complete_path)
            n_atoms_distangdih = specific_file_object.readline()
            n_atoms_distangdih = int(n_atoms_distangdih)

            atoms_list = list(range(1, n_atoms_distangdih + 1))

            print("Which atoms you would like to compute calculations on.")
            print("All you need to type is the atom NUMBER, no spaces (ex. 1,5,7,8,10)")
            print("If you would like all, enter 'all'")

            userAtomsChoice = input("Atoms: ")

            if userAtomsChoice == "all":
                atomsChoice = list(range(1, n_atoms_distangdih + 1))

            else:
                #  remove duplicates using set()
                split_Choices = userAtomsChoice.split(",")

                # removing any possible empty strings
                # https://stackoverflow.com/questions/3845423/remove-empty-strings-from-a-list-of-strings
                split_Choices = list(filter(None, split_Choices))

                atomsChoice = list(map(int, set(split_Choices)))
                atomsChoice.sort()
                #  list(map(int, test_list) will make each number in list an int

            atoms_list = atomsChoice

            print("Would you like to compute distances? Y/N")
            user_comp_distances = input("Distances (Y/N): ").upper()

            if user_comp_distances == "Y":
                print("Distances will be computed")
                comp_distances = True
                print()
                distances_indexes_list = []

                for atom_1 in atoms_list:

                    for atom_2 in atoms_list:
                        if atom_2 > atom_1:
                            distances_indexes_list.append([atom_1, atom_2])

            elif user_comp_distances == "N":
                print("Distances will NOT be computed")
                comp_distances = False
                print()

            else:
                raise InvalidUserEntranceError("Error: Must Choose Y or N")

            print("Would you like to compute angles? Y/N")
            user_comp_angles = input("Angles (Y/N): ").upper()

            if user_comp_angles == "Y":
                print("Angles will be computed")
                comp_angles = True
                print()
                angles_indexes_list = []

                for atom_1 in atoms_list:

                    for atom_2 in atoms_list:
                        if atom_2 != atom_1:

                            for atom_3 in atoms_list:
                                if atom_3 not in [atom_1, atom_2] and atom_3 > atom_1:
                                    angles_indexes_list.append([atom_1,atom_2,atom_3])

            elif user_comp_angles == "N":
                print("Angles will NOT be computed")
                comp_angles = False
                print()

            else:
                raise InvalidUserEntranceError("Error: Must Choose Y or N")

            print("Would you like to compute dihedrals? Y/N")
            user_comp_dihedrals = input("Dihedrals (Y/N): ").upper()

            if user_comp_dihedrals == "Y":
                print("Dihedrals will be computed")
                print()
                comp_dihedrals = True
                dihedral_indexes_list = []

                for atom_1 in atoms_list:

                    for atom_2 in atoms_list:
                        if atom_2 != atom_1:

                            for atom_3 in atoms_list:
                                if atom_3 > atom_2 and atom_3 != atom_1:

                                    for atom_4 in atoms_list:

                                        if atom_4 not in [atom_1, atom_2, atom_3] and atom_4 > atom_1:
                                            dihedral_indexes_list.append([atom_1, atom_2, atom_3, atom_4])

                                            """
                                            if [atom 2, atom 3] is in dihedral_indexes_list (but only position 1 and 2)
                                                                        [____, atom 2, atom 3, ___]
                                            if not any(isinstance([atom_4, atom_2, atom_3, atom_1], dihedral_indexes_list):
                                                test_equality = True
                                                
                                            if [atom 2, atom 3] is in dihedral_indexes_list[][1:2] 
                                            #
                                            # for dihedrals in dihedral_indexes_list:
                                            #     if atom_2 == dihedrals[1] and atom_3 == dihedrals[2] and atom_1 == dihedrals[3] and atom_4 == dihedrals[0]:
                                            #         test_equality = True
                                            #
                                            # if not test_equality:
                                            #     dihedral_indexes_list.append([atom_1, atom_2, atom_3, atom_4])
                                            """

            elif user_comp_dihedrals == "N":
                print("Dihedrals will NOT be computed")
                comp_dihedrals = False
                print()

            else:
                raise InvalidUserEntranceError("Error: Must Choose Y or N")

        except InvalidUserEntranceError as e:
            print()
            print(e.message)
            print("Restarting Questioning...")
            print()
            print("______________________________________________________")
            continue

        except ValueError:
            print()
            print("Value Error. You cannot put a word in list of atoms (integers)")
            print("Try Again...")
            print()
            print("______________________________________________________")
            continue
        break


    for file in xyz_files:
        # print(file)  # test works to open xyz file
        complete_path = user_directory + "/" + file  # need complete path to open file
        specific_file_object = open(complete_path)

        n_atoms = specific_file_object.readline()
        n_atoms = int(n_atoms)
        structure_name = specific_file_object.readline()  # subject to the operation performed prior to exporting XYZ; could change
        temp_xyz_df = specific_file_object.readlines()
        # print(temp_xyz_df)
        temp_xyz_contents_list = []
        temp_xyz_colname_list = []

        xyz_index_list.append(file)  # should be file name, will be used to change index from number to file name
        print("Files Found (updates):",xyz_index_list)
        atom_num = 1

        # XYZ
        #  Getting XYZ Content (Cartesian Coordinates)
        for elementxyzs in temp_xyz_df:
            # print(elementxyzs)
            split_elementxyzs = elementxyzs.split()
            if (len(split_elementxyzs) > 0):  # necessary because at the end of each xyz file, there are three blank lines
                # print(split_elementxyzs)
                # print(split_elementxyzs[0]) atom name/element name
                column_name = str(atom_num) + " " + split_elementxyzs[0]  # ex. 15 C; overall atom number and element

                temp_xyz_contents_list.append(split_elementxyzs[1])
                temp_xyz_colname_list.append(column_name + " -X")
                temp_xyz_contents_list.append(split_elementxyzs[2])
                temp_xyz_colname_list.append(column_name + " -Y")
                temp_xyz_contents_list.append(split_elementxyzs[3])
                temp_xyz_colname_list.append(column_name + " -Z")
                atom_num += 1
        master_xyz_contents_list.append(temp_xyz_contents_list)
        print("Cartesian Cordinates: COMPLETE")

        # DISTANCES
        # Getting Distances from each atom to all others
        if comp_distances:
            temp_dist_list = []
            dist_colname_list = []

            for dist_list in distances_indexes_list:
                a1 = [float(temp_xyz_contents_list[3 * dist_list[0] - 3]),
                      float(temp_xyz_contents_list[3 * dist_list[0] - 2]),
                      float(temp_xyz_contents_list[3 * dist_list[0] - 1])]
                a1_array = np.array(a1)

                a2 = [float(temp_xyz_contents_list[3 * dist_list[1] - 3]),
                      float(temp_xyz_contents_list[3 * dist_list[1] - 2]),
                      float(temp_xyz_contents_list[3 * dist_list[1] - 1])]
                a2_array = np.array(a2)

                v1 = a2_array - a1_array
                dist = np.linalg.norm(v1)
                temp_dist_list.append(dist)
                dist_colname_list.append(temp_xyz_colname_list[3 * dist_list[0] - 1][:-2] +
                                         "to " + temp_xyz_colname_list[3 * dist_list[1] - 1][:-2])  # Ex. 15 C to 18 H

            master_dist_contents_list.append(temp_dist_list)
            calculations_list.append(temp_dist_list)
            print("Distances: COMPLETE")

            # current_index = 0
            # atom_index = n_atoms * 3
            # while current_index < atom_index:
            #     if current_index/3 in atoms_list: #or (current_index/3)+1 in atoms_list:
            #         print(current_index)
            #         print(current_index/3)
            #         print(current_index+1)
            #         print("NEWWWWWWW")
            #         print()
            #
            #
            #         # a1 is the coordinates for one atom; three indices used in temp_xyz_contents to describe one atom
            #         a1 = [float(temp_xyz_contents_list[current_index]),
            #               float(temp_xyz_contents_list[current_index + 1]),
            #               float(temp_xyz_contents_list[current_index + 2])]
            #
            #         index_iteration = current_index + 3  # overall would be index 3 in the first loop
            #         #  index 3 would be the x-coord of a new atom
            #         a1_array = np.array(a1)
            #
            #     else:
            #         current_index = current_index + 3
            #         continue
            #
            #     while index_iteration < atom_index:  # same check as first while, but will catch last index
            #         if index_iteration / 3 in atoms_list or index_iteration + 1 in atoms_list:
            #             a2 = [float(temp_xyz_contents_list[index_iteration]),
            #                   float(temp_xyz_contents_list[index_iteration + 1]),
            #                   float(temp_xyz_contents_list[index_iteration + 2])]
            #
            #             a2_array = np.array(a2)
            #             v1 = a2_array-a1_array
            #             dist = np.linalg.norm(v1)
            #             temp_dist_list.append(dist)
            #             dist_colname_list.append(temp_xyz_colname_list[current_index][:-2] +
            #                                      "to " + temp_xyz_colname_list[index_iteration][:-2])  # Ex. 15 C to 18 H
            #
            #             index_iteration = index_iteration + 3
            #
            #         else:
            #             index_iteration = index_iteration + 3
            #             continue
            #
            #     current_index = current_index + 3


        # ANGLES
        # Getting Angles from each atom to all others

        if comp_angles:
            temp_angle_list = []
            angle_colname_list = []

            for angle_list in angles_indexes_list:
                a1 = [float(temp_xyz_contents_list[3 * angle_list[0] - 3]),
                      float(temp_xyz_contents_list[3 * angle_list[0] - 2]),
                      float(temp_xyz_contents_list[3 * angle_list[0] - 1])]
                a1_array = np.array(a1)

                a2 = [float(temp_xyz_contents_list[3 * angle_list[1] - 3]),
                      float(temp_xyz_contents_list[3 * angle_list[1] - 2]),
                      float(temp_xyz_contents_list[3 * angle_list[1] - 1])]
                a2_array = np.array(a2)

                a3 = [float(temp_xyz_contents_list[3 * angle_list[2] - 3]),
                      float(temp_xyz_contents_list[3 * angle_list[2] - 2]),
                      float(temp_xyz_contents_list[3 * angle_list[2] - 1])]
                a3_array = np.array(a3)

                v1 = a2_array - a1_array
                v2 = a3_array - a2_array
                da = np.arccos(np.dot(v1, v2) / (np.linalg.norm(v1) * np.linalg.norm(v2)))  # in radians
                da_degrees = np.degrees(da)
                temp_angle_list.append(da_degrees)

                angle_colname_list.append(temp_xyz_colname_list[3 * angle_list[0] - 1][:-2] +
                                         "to " + temp_xyz_colname_list[3 * angle_list[1] - 1][:-2]+
                                         "to " + temp_xyz_colname_list[3 * angle_list[2] - 1][:-2])  # Ex. 15 C to 18 H

            master_angle_contents_list.append(temp_angle_list)
            print("Angles: COMPLETE")

            # current_index = 0
            # atom_index = n_atoms*3
            #
            # while current_index < atom_index:
            #     if current_index / 3 in atoms_list or current_index + 1 in atoms_list:
            #         index_iteration = 0
            #
            #         a1 = [float(temp_xyz_contents_list[current_index]),
            #               float(temp_xyz_contents_list[current_index+1]),
            #               float(temp_xyz_contents_list[current_index+2])]
            #         a1_array = np.array(a1)
            #
            #     else:
            #         current_index = current_index+3
            #         continue
            #
            #     while index_iteration < atom_index:
            #
            #         if index_iteration == current_index:  #  ensures no index angle can be compared to itself
            #             index_iteration = index_iteration+3
            #             continue
            #
            #         elif index_iteration / 3 in atoms_list or index_iteration + 1 in atoms_list:
            #             index_iteration2 = 0
            #
            #             a2 = [float(temp_xyz_contents_list[index_iteration]),
            #                   float(temp_xyz_contents_list[index_iteration + 1]),
            #                   float(temp_xyz_contents_list[index_iteration + 2])]
            #             a2_array = np.array(a2)
            #
            #             v1 = a2_array - a1_array
            #
            #         else:
            #             index_iteration = index_iteration + 3
            #             continue
            #
            #         #  Now get v2
            #         while index_iteration2 < atom_index:
            #             if index_iteration2 == index_iteration or index_iteration2 == current_index or index_iteration2 > index_iteration:
            #                 index_iteration2 = index_iteration2 + 3 #same as above, no comparison to self
            #                 continue
            #
            #             elif index_iteration2 / 3 in atoms_list or index_iteration2 + 1 in atoms_list:
            #                 a3 = [float(temp_xyz_contents_list[index_iteration2]),
            #                       float(temp_xyz_contents_list[index_iteration2 + 1]),
            #                       float(temp_xyz_contents_list[index_iteration2 + 2])]
            #                 a3_array = np.array(a3)
            #
            #                 v2 = a3_array - a2_array
            #                 da = np.arccos(np.dot(v1, v2) / (np.linalg.norm(v1) * np.linalg.norm(v2)))  # in radians
            #                 da_degrees = np.degrees(da)
            #                 temp_angle_list.append(da_degrees)
            #                 angle_colname_list.append(temp_xyz_colname_list[current_index][:-2] +
            #                                           "to " + temp_xyz_colname_list[index_iteration][:-2] +
            #                                           "to " + temp_xyz_colname_list[index_iteration2][:-2])
            #                 # Ex. 15 C to 18 H to 20 F
            #                 index_iteration2 = index_iteration2 + 3
            #
            #             else:
            #                 index_iteration2 = index_iteration2 + 3
            #                 continue
            #
            #         index_iteration = index_iteration + 3
            #
            #     current_index = current_index+3



        # Dihedrals
        # Getting Dihedrals from each atom to all others
        if comp_dihedrals:
            temp_dihedral_list = []
            dihedral_colname_list = []

            # current_index = 0
            # atom_index = n_atoms*3
            #
            # while current_index < atom_index:
            #     index_iteration = 0
            #
            #     a1 = [float(temp_xyz_contents_list[current_index]),
            #           float(temp_xyz_contents_list[current_index+1]),
            #           float(temp_xyz_contents_list[current_index+2])]
            #     a1_array = np.array(a1)
            #
            #     while (index_iteration < atom_index):
            #         index_iteration2 = 0
            #
            #         if(index_iteration == current_index):  #  ensures no index angle can be compared to itself
            #             index_iteration = index_iteration+3
            #             continue
            #
            #         else:
            #             a2 = [ float(temp_xyz_contents_list[index_iteration]),
            #                    float(temp_xyz_contents_list[index_iteration+1]),
            #                    float(temp_xyz_contents_list[index_iteration+2])]
            #             a2_array = np.array(a2)
            #
            #             v1 = a2_array-a1_array
            #
            #             #  Now get v2
            #             while(index_iteration2 < atom_index):
            #                 index_iteration3 = 0
            #
            #                 if(index_iteration2 == index_iteration or index_iteration2 == current_index):
            #                     index_iteration2 = index_iteration2 + 3 #same as above, no comparison to self
            #                     continue
            #
            #                 else:
            #
            #                     a3 = [float(temp_xyz_contents_list[index_iteration2]),
            #                           float(temp_xyz_contents_list[index_iteration2 + 1]),
            #                           float(temp_xyz_contents_list[index_iteration2 + 2])]
            #                     a3_array = np.array(a3)
            #
            #                     v2 = a3_array - a2_array
            #                     n1 = np.cross(v1,v2) #  cross product
            #
            #                     while index_iteration3 < atom_index:
            #                         if(index_iteration3 == index_iteration or index_iteration3 == index_iteration2 or
            #                         index_iteration3 == current_index):
            #                             index_iteration3 = index_iteration3 + 3
            #                             continue
            #
            #                         else:
            #                             a4 = [float(temp_xyz_contents_list[index_iteration3]),
            #                                   float(temp_xyz_contents_list[index_iteration3 + 1]),
            #                                   float(temp_xyz_contents_list[index_iteration3 + 2])]
            #                             a4_array = np.array(a4)
            #
            #                             v3 = a2_array - a3_array
            #                             v4 = a4_array - a3_array
            #                             n2 = np.cross(v3,v4)
            #
            #                             dihedral = np.arccos(np.round(-np.dot(n1, n2) /
            #                                                           (np.linalg.norm(n1) *
            #                                                            np.linalg.norm(n2)), decimals=4)) #rads
            #
            #                             # dihedral = np.arccos(-np.dot(n1, n2)/(np.linalg.norm(n1)*np.linalg.norm(n2))) # rads
            #
            #                             # Hold for demonstration on friday: to revert code, all you need
            #                             # to do is to remove the np.round part (and decimals =) to show how the rounding
            #                             # was messing up everything
            #                             # if math.isnan(dihedral):
            #                             #     print("n1:", n1)
            #                             #     print("n2:", n2)
            #                             #     print("dihedral:",dihedral)
            #                             #     print("dotproduct n1n2:",-np.dot(n1,n2))
            #                             #     print("n1 norm:",np.linalg.norm(n1))
            #                             #     print("n2 norm:",np.linalg.norm(n2))
            #                             #
            #                             #     print("not rounded:",-np.dot(n1,n2)/(np.linalg.norm(n1)*np.linalg.norm(n2)))
            #                             #     print(np.round(-np.dot(n1,n2)/(np.linalg.norm(n1)*np.linalg.norm(n2)),decimals=3))
            #                             #     sys.exit()
            #                             #     break
            #
            #                             if np.dot(n1,v4) > 0:
            #                                 dihedral = -1*dihedral
            #
            #                             dihedral_degrees = np.degrees(dihedral)
            #
            #                             #MODIFICATION OF DIHEDRALS (NO LONGER ACTUAL DIHEDRALS)
            #                             if dihedral_degrees > 90 and dihedral_degrees < 180:
            #                                 dihedral_degrees = 180 - dihedral_degrees
            #
            #                             elif dihedral_degrees < -90 and dihedral_degrees > -180:
            #                                 dihedral_degrees = -180-dihedral_degrees
            #
            #                             #  Its important to note that dihedrals have been modified to
            #                             #  ensure PCA treats dihedrals of different sinage similarly
            #                             #  we want PCA to know that the maximal difference in dihedrals
            #                             #  is dependent on whether or not a dihedral is close to 90 degrees
            #
            #                             temp_dihedral_list.append(dihedral_degrees)
            #
            #                             dihedral_colname_list.append(temp_xyz_colname_list[current_index][:-2] +
            #                                                          "to " + temp_xyz_colname_list[index_iteration][:-2] +
            #                                                          "to " + temp_xyz_colname_list[index_iteration2][:-2] +
            #                                                          "to " + temp_xyz_colname_list[index_iteration3][:-2])
            #                             # Ex. 1 C to 12 H to 20 F to 21 C
            #
            #                             index_iteration3 = index_iteration3 +3
            #
            #                 index_iteration2 = index_iteration2 + 3
            #
            #
            #             index_iteration = index_iteration + 3
            #
            #     current_index = current_index+3
            # master_dihedral_contents_list.append(temp_dihedral_list)

            for di_list in dihedral_indexes_list:
                a1 = [float(temp_xyz_contents_list[3*di_list[0]-3]),
                      float(temp_xyz_contents_list[3*di_list[0]-2]),
                      float(temp_xyz_contents_list[3*di_list[0]-1])]
                a1_array = np.array(a1)

                a2 = [float(temp_xyz_contents_list[3 * di_list[1] - 3]),
                      float(temp_xyz_contents_list[3 * di_list[1] - 2]),
                      float(temp_xyz_contents_list[3 * di_list[1] - 1])]
                a2_array = np.array(a2)

                a3 = [float(temp_xyz_contents_list[3 * di_list[2] - 3]),
                      float(temp_xyz_contents_list[3 * di_list[2] - 2]),
                      float(temp_xyz_contents_list[3 * di_list[2] - 1])]
                a3_array = np.array(a3)

                a4 = [float(temp_xyz_contents_list[3 * di_list[3] - 3]),
                      float(temp_xyz_contents_list[3 * di_list[3] - 2]),
                      float(temp_xyz_contents_list[3 * di_list[3] - 1])]
                a4_array = np.array(a4)

                v1 = a2_array - a1_array
                v2 = a3_array - a2_array
                n1 = np.cross(v1, v2)  # cross product
                v3 = a2_array - a3_array
                v4 = a4_array - a3_array
                n2 = np.cross(v3, v4)


                dihedral = np.arccos(np.round(-np.dot(n1, n2)/(np.linalg.norm(n1) * np.linalg.norm(n2)), decimals=4))
                # radians ^

                if np.dot(n1, v4) > 0:
                    dihedral = -1 * dihedral

                dihedral_degrees = np.degrees(dihedral)

                # MODIFICATION OF DIHEDRALS (NO LONGER ACTUAL DIHEDRALS)
                if 90 < dihedral_degrees < 180:
                    dihedral_degrees = 180 - dihedral_degrees

                elif -90 > dihedral_degrees > -180:
                    dihedral_degrees = -180 - dihedral_degrees

                temp_dihedral_list.append(dihedral_degrees)

                dihedral_colname_list.append(temp_xyz_colname_list[3 * di_list[0]-1][:-2] +
                                             "to " + temp_xyz_colname_list[3 * di_list[1]-1][:-2] +
                                             "to " + temp_xyz_colname_list[3 * di_list[2]-1][:-2] +
                                             "to " + temp_xyz_colname_list[3 * di_list[3]-1][:-2])
                # Ex. 1 C to 12 H to 20 F to 21 C
            master_dihedral_contents_list.append(temp_dihedral_list)

            print("Dihedrals: COMPLETE")
            print()
            print()

    test_xyz_df = pd.DataFrame(master_xyz_contents_list,
                               columns=temp_xyz_colname_list)
    test_xyz_df.index = xyz_index_list


    if comp_distances:
        test_dist_df = pd.DataFrame(master_dist_contents_list,
                                    columns=dist_colname_list)
        test_dist_df.index = xyz_index_list

    if comp_angles:
        test_angle_df = pd.DataFrame(master_angle_contents_list,
                                     columns=angle_colname_list)
        test_angle_df.index = xyz_index_list

    if comp_dihedrals:
        test_dihedral_df = pd.DataFrame(master_dihedral_contents_list,
                                     columns=dihedral_colname_list)
        test_dihedral_df.index = xyz_index_list

    if comp_distances and comp_angles and comp_dihedrals:
        frames = [test_dist_df, test_angle_df, test_dihedral_df]

    elif comp_distances and comp_angles:
        frames = [test_dist_df, test_angle_df]

    elif comp_distances and comp_dihedrals:
        frames = [test_dist_df, test_dihedral_df]

    elif comp_angles and comp_dihedrals:
        frames = [test_angle_df, test_dihedral_df]

    elif comp_distances:
        frames = [test_dist_df]

    elif comp_angles:
        frames = [test_angle_df]

    elif comp_dihedrals:
        frames = [test_dihedral_df]

    calculations_df = pd.concat(frames, axis=1)
    specific_file_object.close()
    return calculations_df
    # return test_xyz_df,test_dist_df,test_angle_df,test_dihedral_df,calculations_df



from sklearn.decomposition import PCA
import numpy as np
import plotly.express as px
import pandas as pd
import os

# column_headers = [" 1 C to 2 H", "1 C to 3 H", "1 C to 4 C", "17 H to 16 H to 15 H to 12 H",
#                "17 H to 16 H to 15 H to 13 H", "17 H to 16 H to 15 H to 14 C"]
#
# indexes = ["pentane_1.xyz", "pentane_3.xyz", "pentane_2.xyz", "pentane_4_fake.xyz", "pentane_5_fake.xyz"]
#
#
# A = np.array([[1.096276,1.096276,1.528389,55.236766,71.552750,37.123885],
#               [1.096818,1.095462,1.530606,121.597386,100.494262,37.106736],
#               [1.097144,1.095743,1.531324,67.610414,128.966981,37.456963],
#               [1.530606,1.095462,1.196818,121.106736,100.966981,37.530606],
#               [128.966981,1.095743,1.531324,37.456963,1.097144,67.610414]])
#
#
# test_df = pd.DataFrame(A, columns=column_headers)
# test_df.index = indexes
# print(test_df)
# print(test_df.shape[0])



###need to create try catch block with input of  user PCs
# if the user asks for too high of pc then kick them back to the top of the while or for loop and renter number
# if user ask >= test_df.shape[0], then change it jackass!!!

class InvalidUserEntranceError(Exception):
    def __init__(self, message):
        self.message = message


def pca_model(user_input_pca,dataframe):
    model = PCA(n_components=user_input_pca)
    print()
    print("______________________________________________________")
    print("Model created with {} principal component(s)".format(user_input_pca))
    print("Default plots to be shown")
    print("______________________________________________________")
    num_components = user_input_pca
    components = model.fit_transform(dataframe)
    # print(components)
    size_dots = dataframe.shape[0]*[float(00000.1)]
    size_dots = range(1,23)

    #Explained Variance Plot
    explained_variance = np.cumsum(model.explained_variance_ratio_)
    fig1 = px.area(x = range(1, explained_variance.shape[0] + 1), y = explained_variance,
                   labels={"x": "Number of Components", "y":"Explained Variance"})

    fig1.show() #BE SURE TO UNCOMMENT AT END

    if  user_input_pca == 2:
        # Scatter Plot
        fig2 = px.scatter(components, x=0, y=1, hover_name=dataframe.index, labels={"0": "PC 1", "1": "PC 2"},size_max=0.1)#, size = size_dots)
        fig2.show()

    elif user_input_pca == 3:
        total_var = model.explained_variance_ratio_.sum() * 100

        pca_labels = {str(i): f"PC {i + 1} ({var:.1f}%)" for i, var in enumerate(model.explained_variance_ratio_ * 100)}
        fig3 = px.scatter_matrix(components, labels=pca_labels,
                                 dimensions= range(user_input_pca),
                                 hover_name=dataframe.index, size_max=0.1)#,
                                 #size=size_dots)
        fig3.update_traces(diagonal_visible = False)
        fig3.show()

        fig4 = px.scatter_3d(components, x=0, y=1, z=2,
                             title=f'Total Explained Variance: {total_var:.2f}%',
                             labels= {"0": "PC 1", "1": "PC 2", "2": "PC 3"},
                             hover_name=dataframe.index,size_max=0.1)#,
                             #size=size_dots)
        fig4.show()

    else:
        print("You have asked for a not-optimal number of components for visualization ( >3 dimensions ) ")
        total_var = model.explained_variance_ratio_.sum() * 100

        pca_labels = {str(i): f"PC {i + 1} ({var:.1f}%)" for i, var in enumerate(model.explained_variance_ratio_ * 100)}
        fig3 = px.scatter_matrix(components, labels=pca_labels,
                                 dimensions= range(user_input_pca),
                                 hover_name=dataframe.index,
                                 title=f'Total Explained Variance: {total_var:.2f}%',
                                 )
        fig3.update_traces(diagonal_visible = False)
        fig3.show()

    attempts_left = 10
    for attempt in range(10):  # user has ten times to get a correct num of pcs desired, otherwise must restart
        try:
            print()
            print("Would you like to see which features in the original dataset contribute the most to a chosen PC?")

            feature_importance_test = input("Enter Y/N: ").upper()

            while feature_importance_test != "Y" and feature_importance_test != "N":
                raise InvalidUserEntranceError("You did not choose Y or N, try again.")

            if feature_importance_test == "Y":
                continue_feature_insights = True

                while continue_feature_insights:
                    print("Which Principal Component would you like to test?")
                    test_PC = int(input("PC #: "))

                    while test_PC > user_input_pca:
                        raise InvalidUserEntranceError("Error; PC not calculated due to previous input response. Try Again")

                    print()
                    print("PC Noted--Finding Feature Importance")
                    print("The top 10 features with the largest influcence on PC", test_PC, "will be printed")
                    print()

                    # model.components_ are the loadings, which give a magnitude and direction on the PC given
                    # model.components_[test_PC - 1] --> list of components

                    loading_scores = pd.Series(model.components_[test_PC - 1],
                                               index=dataframe.columns.tolist())

                    sorted_loading_scores = loading_scores.abs().sort_values(
                        ascending=False)  # contains all loading scores

                    top10_features = sorted_loading_scores[0:10].index.values

                    print("Top 10 Features and their given magnitude and direction")
                    print(loading_scores[top10_features])
                    print()
                    print("Would you like to stop?")
                    user_stop_feat = input("Enter Y/N: ").upper()

                    if user_stop_feat == "Y":
                        continue_feature_insights = False

            else:
                print()
                print("You chose not to see the features, continuing...")

            print("No longer seeing features, continuing...")

        except InvalidUserEntranceError as e:
            print()
            print(e.message)
            print()
            print("______________________________________________________")
            attempts_left -= 1
            print(attempts_left, "attempts left")
            continue

        except ValueError:
            print()
            print("Not an integer")
            attempts_left -= 1
            print(attempts_left, "attempts left")
            continue

        break

    return components,num_components,dataframe

#  dataframe = original dataframe



import os
import xyz
import pca_creation
import pymol
import clustering
import stat
from datetime import datetime

startTime = datetime.now()
print("What is the directory in which your xyz files are in?")

class PathNoExistError(Exception):
    def __init__(self, message):
        self.message = message


class NoXYZFileExistError(Exception):
    def __init__(self, message):
        self.message = message


class InvalidPCAError(Exception):
    def __init__(self, message):
        self.message = message


class InvalidClusterError(Exception):
    def __init__(self, message):
        self.message = message


class InvalidUserEntranceError(Exception):
    def __init__(self, message):
        self.message = message


class ConformationalAnalysis(object):

    attempts_left = 10
    for attempt in range(10):  # user has ten times to get a correct path, otherwise must restart
        try:
            user_input_directory = input("XYZ File Directory Path: ")
            # user_input_directory = "/Users/matthewnwerem/Chapman University/OMO Research Group - Project Conf. Analysis - Nwerem - proj_conf_analysis/Code Versions/Conformational_Analysis/test_files/cycloheptadecane_file_test2"
            # user_input_directory = "/Users/matthewnwerem/Chapman University/OMO Research Group - Project Conf. Analysis - Nwerem - proj_conf_analysis/Code Versions/Conformational_Analysis/test_files/cycloheptadecane_file_test"
            # user_input_directory = "/Users/matthewnwerem/Chapman University/OMO Research Group - Project Conf. Analysis - Nwerem - proj_conf_analysis/Code Versions/Conformational_Analysis/test_files/cyclooctane_file_test"
            # user_input_directory = "/Users/matthewnwerem/Chapman University/OMO Research Group - Project Conf. Analysis - Nwerem - proj_conf_analysis/Code Versions/Conformational_Analysis/test_files/pentane_file_test"
            # user_input_directory = "/Users/matthewnwerem/Chapman University/OMO Research Group - Project Conf. Analysis - Nwerem - proj_conf_analysis/Code Versions/Conformational_Analysis/test_files/xyz_file_test"
            # user_input_directory = "/Users/matthewnwerem/Chapman University/OMO Research Group - Project Conf. Analysis - Nwerem - proj_conf_analysis/Code Versions/Conformational_Analysis/test_files/1_4_dioxepane_file_test"
            # user_input_directory = "/Users/matthewnwerem/Chapman University/OMO Research Group - Project Conf. Analysis - Nwerem - proj_conf_analysis/Code Versions/Conformational_Analysis/test_files/1fluoro4propylcyclohexane"
            # user_input_directory = "/Users/matthewnwerem/Chapman University/OMO Research Group - Project Conf. Analysis - Nwerem - proj_conf_analysis/Code Versions/Conformational_Analysis/test_files/cis_1fluoro4propylcyclohexane"

            # 1,4,7,9,12,14,18,21,24,27,30,33,36,39,42,45,46

            while not os.path.exists(user_input_directory):
                raise PathNoExistError("Path: '{}' does not exist".format(user_input_directory))

            print("______________________________________________________")
            print(user_input_directory)
            print("Path Received")
            print("______________________________________________________")

            directory_files = os.listdir(user_input_directory)
            xyz_files = []

            for file in directory_files:
                if file.endswith(".xyz"):
                    xyz_files.append(file)

            if not xyz_files:  # means if the list is empty (using implicit booleanness of empty lists)
                raise NoXYZFileExistError("There are no XYZ files in the path '{}".format(user_input_directory))

            # print("Current Test XYZ File:", xyz_files[0]) #0 because first index of a LIST
            # print("Type w/ [0]", type(xyz_files[0]))
            # print("Type w/o [0]", type(xyz_files))

            # print(xyz_files)
            # complete_path = user_input_directory+"/"+xyz_files[0] #need complete path to open file
            # specific_file_object = open(complete_path)  # will need to make loop for each xyzfile when doing multiple i think

            xyz_df = xyz.xyz_contents(xyz_files, user_input_directory)
            print("______________________________________________________")
            print("Current Dataframe")
            print("Has been tested and works with")
            print("1: Adding all xyz's of t-amlyOH1.xyz from a folder to a dataframe")
            print("2: Adding a second xyz (just another copy of amylOH_1) to the same dataframe ")
            print("3: Calculating distances angles and dihedrals")
            print("4: Adding distances, angles and dihedrals to dataframe")
            print("______________________________________________________")
            print(xyz_df)
            print(datetime.now() - startTime)


        except PathNoExistError as e:
            print()
            print(e.message)
            print()
            print("______________________________________________________")
            attempts_left -= 1
            print(attempts_left, "attempts left")
            continue
        except NoXYZFileExistError as e:
            print()
            print(e.message)
            print()
            print("______________________________________________________")
            attempts_left -= 1
            print(attempts_left, "attempts left")
            continue

        break

    attempts_left = 10
    print("______________________________________________________")
    print("PCA")
    print("Has been tested and works with")
    print("1: Creating PCA Model and its subsequent plots")
    print("2: all interactive plots show correct hover name ")
    print("3: showing top 10 features that effect the PC")
    print("______________________________________________________")

    for attempt in range(10):  # user has ten times to get a correct num of pcs desired, otherwise must restart
        try:
            testPCs = True
            while testPCs:
                userPCA_Components = input("Enter the number of components you would like to see in the model: ")
                userPCA_Components = int(userPCA_Components)

                while userPCA_Components >= xyz_df.shape[0]:
                    raise InvalidPCAError("The # of components must be less than {}".format(xyz_df.shape[0]))

                userPCA_model, user_num_components,original_df = pca_creation.pca_model(userPCA_Components, xyz_df)

                print()
                print("Would you like to test with a different number of PCs?")
                testing_PCs = input("Enter Y/N: ").upper()
                print()

                if testing_PCs == "Y":
                    testPCs = True

                elif testing_PCs == "N":
                    testPCs = False
                    print("Ending...")

                while testing_PCs != "Y" and testing_PCs != "N":
                    raise InvalidUserEntranceError("You did not choose Y or N, try again.")

        except InvalidPCAError as e:
            print()
            print(e.message)
            print("Your number of components desired and dataframe dimensions do not work well for PCA")
            print()
            print("______________________________________________________")
            attempts_left -= 1
            print(attempts_left, "attempts left")
            continue

        except ValueError:
            print()
            print("Not an integer")
            attempts_left -= 1
            print(attempts_left, "attempts left")
            continue

        except InvalidUserEntranceError as e:
            print()
            print(e.message)
            print()
            print("______________________________________________________")
            attempts_left -= 1
            print(attempts_left, "attempts left")
            continue

        break

    attempts_left = 10
    print("______________________________________________________")
    print("Clustering")
    print("Has been tested and works with")
    print("1: Creating Clustering Model with PCA data and its subsequent plots")
    print("2: All interactive plots show correct hover name ")
    print("______________________________________________________")

    for attempt in range(10):  # user has ten times to get a correct num of pcs desired, otherwise must restart
        try:
            testClusters = True
            while testClusters:
                tested_kclusters = 10

                # see if you can do it in a way where it depends on the number of conformations brought in
                # aka like change tested clusters to be half of the shape of all. That way its better
                # userPCA_model.shape[0]/2 ?? or userPCA_model.shape[0]/5?? test out and see whats best

                while tested_kclusters >= userPCA_model.shape[0]:
                    raise InvalidClusterError("The # of clusters to be tested is greater than {}".format(userPCA_model.shape[0]))
                    #  if this is raised, it means that you are looking into less than 10 structures
                    #  in that case, it will be tougher to find differentiation anyway

                userClustermodel,num_clusters_user = clustering.clustering_model(userPCA_model, user_num_components,original_df)

                print()
                print("Would you like to test with a different number of Clusters?")
                testing_Clusters = input("Enter Y/N: ").upper()
                print()

                if testing_Clusters == "Y":
                    testClusters = True

                elif testing_Clusters == "N":
                    testClusters = False
                    print("Ending...")

                while testing_Clusters != "Y" and testing_Clusters != "N":
                    raise InvalidUserEntranceError("You did not choose Y or N, try again.")

        except InvalidClusterError as e:
            print()
            print(e.message)
            print("Your number of components desired and dataframe dimensions do not work well for PCA")
            print()
            print("______________________________________________________")
            attempts_left -= 1
            print(attempts_left, "attempts left")
            continue

        # except ValueError:
        #     print()
        #     print("Not an integer")
        #     attempts_left -= 1
        #     print(attempts_left, "attempts left")
        #     continue

        except InvalidUserEntranceError as e:
            print()
            print(e.message)
            print()
            print("______________________________________________________")
            attempts_left -= 1
            print(attempts_left, "attempts left")
            continue

        break

    attempts_left = 10
    # os.chmod('mGenerate_PyMOL_Session.sh', stat.S_IRWXU)
    print("______________________________________________________")
    print("Sorting files by family")
    print("After reviewing the given plots, which of the files would you like to view in pyMol?")
    print("Enter the filename with  (ex. 'tamylOH_17.xyz'), with commas in between each name")
    print("______________________________________________________")

    for attempt in range(10):  # user has ten times to get a correct num of pcs desired, otherwise must restart
        try:
            pymolSession = True
            current_directory_overall = os.getcwd()

            while pymolSession:

                # userPymolSession = pymol.generate_pymol(user_input_directory)
                user_FileFamilies = pymol.family_files(userClustermodel,num_clusters_user,user_input_directory)
                print()
                print("Would you like to generate a new file structure of your families?")
                new_PyMolSession = input("Enter Y/N: ").upper()
                print()

                if new_PyMolSession == "Y":
                    os.chdir(current_directory_overall)
                    pymolSession = True

                elif new_PyMolSession == "N":
                    pymolSession = False
                    print("Ending...")

                while new_PyMolSession != "Y" and new_PyMolSession != "N":
                    raise InvalidUserEntranceError("You did not choose Y or N, try again.")

        except InvalidUserEntranceError as e:
            print()
            print(e.message)
            print()
            print("______________________________________________________")
            attempts_left -= 1
            print(attempts_left, "attempts left")
            continue

        break









# Create dataframe with xyz contents
import pandas as pd
import numpy as np
import elements
import sys
import math
master_xyz_contents_list = []
master_dist_contents_list = []
master_angle_contents_list = []
master_dihedral_contents_list = []
calculations_list = []


class InvalidUserEntranceError(Exception):
    def __init__(self, message):
        self.message = message


def calc_distances(distances_indexes_list,temp_xyz_contents_list,temp_xyz_colname_list):
    temp_dist_list = []
    dist_colname_list = []

    for dist_list in distances_indexes_list:
        a1 = [float(temp_xyz_contents_list[3 * dist_list[0] - 3]),
              float(temp_xyz_contents_list[3 * dist_list[0] - 2]),
              float(temp_xyz_contents_list[3 * dist_list[0] - 1])]
        a1_array = np.array(a1)

        a2 = [float(temp_xyz_contents_list[3 * dist_list[1] - 3]),
              float(temp_xyz_contents_list[3 * dist_list[1] - 2]),
              float(temp_xyz_contents_list[3 * dist_list[1] - 1])]
        a2_array = np.array(a2)

        v1 = a2_array - a1_array
        dist = np.linalg.norm(v1)
        temp_dist_list.append(dist)
        dist_colname_list.append(temp_xyz_colname_list[3 * dist_list[0] - 1][:-2] +
                                 "to " + temp_xyz_colname_list[3 * dist_list[1] - 1][:-2])  # Ex. 15 C to 18 H

    master_dist_contents_list.append(temp_dist_list)
    calculations_list.append(temp_dist_list)
    print("Distances: COMPLETE")
    return master_dist_contents_list,dist_colname_list


def calc_angles(angles_indexes_list,temp_xyz_contents_list,temp_xyz_colname_list):
    temp_angle_list = []
    angle_colname_list = []

    for angle_list in angles_indexes_list:
        a1 = [float(temp_xyz_contents_list[3 * angle_list[0] - 3]),
              float(temp_xyz_contents_list[3 * angle_list[0] - 2]),
              float(temp_xyz_contents_list[3 * angle_list[0] - 1])]
        a1_array = np.array(a1)

        a2 = [float(temp_xyz_contents_list[3 * angle_list[1] - 3]),
              float(temp_xyz_contents_list[3 * angle_list[1] - 2]),
              float(temp_xyz_contents_list[3 * angle_list[1] - 1])]
        a2_array = np.array(a2)

        a3 = [float(temp_xyz_contents_list[3 * angle_list[2] - 3]),
              float(temp_xyz_contents_list[3 * angle_list[2] - 2]),
              float(temp_xyz_contents_list[3 * angle_list[2] - 1])]
        a3_array = np.array(a3)

        v1 = a2_array - a1_array
        v2 = a3_array - a2_array
        da = np.arccos(np.dot(v1, v2) / (np.linalg.norm(v1) * np.linalg.norm(v2)))  # in radians
        da_degrees = np.degrees(da)
        temp_angle_list.append(da_degrees)

        angle_colname_list.append(temp_xyz_colname_list[3 * angle_list[0] - 1][:-2] +
                                  "to " + temp_xyz_colname_list[3 * angle_list[1] - 1][:-2] +
                                  "to " + temp_xyz_colname_list[3 * angle_list[2] - 1][:-2])  # Ex. 15 C to 18 H

    master_angle_contents_list.append(temp_angle_list)
    print("Angles: COMPLETE")
    return master_angle_contents_list,angle_colname_list


def calc_dihedrals(dihedral_indexes_list,temp_xyz_contents_list,temp_xyz_colname_list):
    temp_dihedral_list = []
    dihedral_colname_list = []

    for di_list in dihedral_indexes_list:
        a1 = [float(temp_xyz_contents_list[3 * di_list[0] - 3]),
              float(temp_xyz_contents_list[3 * di_list[0] - 2]),
              float(temp_xyz_contents_list[3 * di_list[0] - 1])]
        a1_array = np.array(a1)

        a2 = [float(temp_xyz_contents_list[3 * di_list[1] - 3]),
              float(temp_xyz_contents_list[3 * di_list[1] - 2]),
              float(temp_xyz_contents_list[3 * di_list[1] - 1])]
        a2_array = np.array(a2)

        a3 = [float(temp_xyz_contents_list[3 * di_list[2] - 3]),
              float(temp_xyz_contents_list[3 * di_list[2] - 2]),
              float(temp_xyz_contents_list[3 * di_list[2] - 1])]
        a3_array = np.array(a3)

        a4 = [float(temp_xyz_contents_list[3 * di_list[3] - 3]),
              float(temp_xyz_contents_list[3 * di_list[3] - 2]),
              float(temp_xyz_contents_list[3 * di_list[3] - 1])]
        a4_array = np.array(a4)

        v1 = a2_array - a1_array
        v2 = a3_array - a2_array
        n1 = np.cross(v1, v2)  # cross product
        v3 = a2_array - a3_array
        v4 = a4_array - a3_array
        n2 = np.cross(v3, v4)

        dihedral = np.arccos(np.round(-np.dot(n1, n2) / (np.linalg.norm(n1) * np.linalg.norm(n2)), decimals=4))
        # radians ^

        if np.dot(n1, v4) > 0:
            dihedral = -1 * dihedral

        dihedral_degrees = np.degrees(dihedral)

        # MODIFICATION OF DIHEDRALS (NO LONGER ACTUAL DIHEDRALS)
        if 90 < dihedral_degrees < 180:
            dihedral_degrees = 180 - dihedral_degrees

        elif -90 > dihedral_degrees > -180:
            dihedral_degrees = -180 - dihedral_degrees

        temp_dihedral_list.append(dihedral_degrees)

        dihedral_colname_list.append(temp_xyz_colname_list[3 * di_list[0] - 1][:-2] +
                                     "to " + temp_xyz_colname_list[3 * di_list[1] - 1][:-2] +
                                     "to " + temp_xyz_colname_list[3 * di_list[2] - 1][:-2] +
                                     "to " + temp_xyz_colname_list[3 * di_list[3] - 1][:-2])
        # Ex. 1 C to 12 H to 20 F to 21 C
    master_dihedral_contents_list.append(temp_dihedral_list)
    print("Dihedrals: COMPLETE")
    print()
    print()

    return master_dihedral_contents_list,dihedral_colname_list

def calc_dihedrals_new(proper_dihedrals,temp_xyz_contents_list,temp_xyz_colname_list):
    temp_dihedral_list = []
    dihedral_colname_list = []

    for di_list in proper_dihedrals:
        a1 = [float(temp_xyz_contents_list[3 * di_list[0].atom_num - 3]),
              float(temp_xyz_contents_list[3 * di_list[0].atom_num - 2]),
              float(temp_xyz_contents_list[3 * di_list[0].atom_num - 1])]
        a1_array = np.array(a1)

        a2 = [float(temp_xyz_contents_list[3 * di_list[1].atom_num - 3]),
              float(temp_xyz_contents_list[3 * di_list[1].atom_num - 2]),
              float(temp_xyz_contents_list[3 * di_list[1].atom_num - 1])]
        a2_array = np.array(a2)

        a3 = [float(temp_xyz_contents_list[3 * di_list[2].atom_num - 3]),
              float(temp_xyz_contents_list[3 * di_list[2].atom_num - 2]),
              float(temp_xyz_contents_list[3 * di_list[2].atom_num - 1])]
        a3_array = np.array(a3)

        a4 = [float(temp_xyz_contents_list[3 * di_list[3].atom_num - 3]),
              float(temp_xyz_contents_list[3 * di_list[3].atom_num - 2]),
              float(temp_xyz_contents_list[3 * di_list[3].atom_num - 1])]
        a4_array = np.array(a4)

        v1 = a2_array - a1_array
        v2 = a3_array - a2_array
        n1 = np.cross(v1, v2)  # cross product
        v3 = a2_array - a3_array
        v4 = a4_array - a3_array
        n2 = np.cross(v3, v4)

        dihedral = np.arccos(np.round(-np.dot(n1, n2) / (np.linalg.norm(n1) * np.linalg.norm(n2)), decimals=4))
        # radians ^

        if np.dot(n1, v4) > 0:
            dihedral = -1 * dihedral

        dihedral_degrees = np.degrees(dihedral)

        # MODIFICATION OF DIHEDRALS (NO LONGER ACTUAL DIHEDRALS)
        if 90 < dihedral_degrees < 180:
            dihedral_degrees = 180 - dihedral_degrees

        elif -90 > dihedral_degrees > -180:
            dihedral_degrees = -180 - dihedral_degrees

        temp_dihedral_list.append(dihedral_degrees)

        dihedral_colname_list.append(temp_xyz_colname_list[3 * di_list[0].atom_num - 1][:-2] +
                                     "to " + temp_xyz_colname_list[3 * di_list[1].atom_num - 1][:-2] +
                                     "to " + temp_xyz_colname_list[3 * di_list[2].atom_num - 1][:-2] +
                                     "to " + temp_xyz_colname_list[3 * di_list[3].atom_num - 1][:-2])
        # Ex. 1 C to 12 H to 20 F to 21 C
    master_dihedral_contents_list.append(temp_dihedral_list)
    print("Dihedrals: COMPLETE")
    print()
    print()

    return master_dihedral_contents_list,dihedral_colname_list

#  Goal 1: Grab XYZ Content

def xyz_contents(xyz_files, user_directory):
    xyz_index_list = []
    comp_distances = False
    comp_angles = False
    comp_dihedrals = False
    atoms_list = None

    while True:
        try:
            complete_path = user_directory + "/" + xyz_files[0]
            specific_file_object = open(complete_path)
            n_atoms_distangdih = specific_file_object.readline()
            n_atoms_distangdih = int(n_atoms_distangdih)
            structure_name = specific_file_object.readline()
            coordinates_structure = specific_file_object.readlines()
            carbon_list = []
            hydrogen_list = []
            oxygen_list = []
            nitrogen_list = []
            fluorine_list = []
            listofatoms = []

            atoms_list = list(range(1, n_atoms_distangdih + 1))

            print("Which atoms you would like to compute calculations on.")
            print("All you need to type is the atom NUMBER, no spaces (ex. 1,5,7,8,10)")
            print("If you would like all, enter 'all'")

            userAtomsChoice = input("Atoms: ")

            #[s for s in mylist if s.isdigit()]
            # digits_Choices = [s for s in split_Choices if s.isdigit()]

            split_choices_atom_num = userAtomsChoice.replace(',', '')
            split_choices_atom_num = str(split_choices_atom_num)

            if userAtomsChoice.lower() == "all":  # option 1: all atoms will be used
                print("You chose to include all atoms")
                atomsChoice = list(range(1, n_atoms_distangdih + 1))

            elif split_choices_atom_num.isdigit(): # option 2: atoms selected by atom number will be used
                #  remove duplicates using set()
                print("You chose to include atoms via atom number")
                split_Choices = userAtomsChoice.split(",")

                # removing any possible empty strings
                # https://stackoverflow.com/questions/3845423/remove-empty-strings-from-a-list-of-strings
                split_Choices = list(filter(None, split_Choices))

                atomsChoice = list(map(int, set(split_Choices)))
                atomsChoice.sort()
                #  list(map(int, test_list) will make each number in list an int


            else: # option 3: atoms types only will be used
                print("You chose to include atoms via element")
                userAtomsChoice = userAtomsChoice.upper() # ensure that inputs are uppercase
                atom_number = 1
                atomsChoice = []

                for line in coordinates_structure:
                    print(line)
                    split_line = line.split()
                    userAtoms_splitChoices = userAtomsChoice.split(",")

                    if split_line[0] in userAtoms_splitChoices:
                        print("True")
                        print(split_line[0])
                        atomsChoice.append(atom_number)
                        print(atomsChoice)
                        atom_number = atom_number + 1

                    else:
                        atom_number = atom_number + 1

                    #     if split_line[0] == "C":
                    #         # print("C atom found")
                    #         carbon_instance = elements.Carbon(atomnum=atom_number)
                    #         carbon_instance.cartesian = [split_line[1], split_line[2], split_line[3]]
                    #         carbon_list.append(carbon_instance)
                    #         listofatoms.append(carbon_instance)
                    #         atoms_list.append(atom_number)
                    #         atom_number = atom_number + 1
                    #
                    #     elif split_line[0] == "H":
                    #         # print("H atom found")
                    #         hydrogen_instance = elements.Hydrogen(atomnum=atom_number)
                    #         hydrogen_instance.cartesian = [split_line[1], split_line[2], split_line[3]]
                    #         hydrogen_list.append(hydrogen_instance)
                    #         listofatoms.append(hydrogen_instance)
                    #         atoms_list.append(atom_number)
                    #         atom_number = atom_number + 1
                    #
                    #     elif split_line[0] == "O":
                    #         # print("O atom found")
                    #         oxygen_instance = elements.Oxygen(atomnum=atom_number)
                    #         oxygen_instance.cartesian = [split_line[1], split_line[2], split_line[3]]
                    #         oxygen_list.append(oxygen_instance)
                    #         listofatoms.append(oxygen_instance)
                    #         atoms_list.append(atom_number)
                    #         atom_number = atom_number + 1
                    #
                    #     elif split_line[0] == "N":
                    #         # print("N atom found")
                    #         nitrogen_instance = elements.Nitrogen(atomnum=atom_number)
                    #         nitrogen_instance.cartesian = [split_line[1], split_line[2], split_line[3]]
                    #         nitrogen_list.append(nitrogen_instance)
                    #         listofatoms.append(nitrogen_instance)
                    #         atoms_list.append(atom_number)
                    #         atom_number = atom_number + 1
                    #
                    #     elif split_line[0] == "F":
                    #         # print("N atom found")
                    #         fluorine_instance = elements.Fluorine(atomnum=atom_number)
                    #         fluorine_instance.cartesian = [split_line[1], split_line[2], split_line[3]]
                    #         fluorine_list.append(fluorine_instance)
                    #         listofatoms.append(fluorine_instance)
                    #         atoms_list.append(atom_number)
                    #         atom_number = atom_number + 1
                    #
                    # print(atoms_list)


            '''
                  if userAtomsChioce is a list of elements (C,H,N) (option 2: using elements only)
                      do same reading as below for one molecule;

                      eg.
                      atom_number = 1
                      If split_line[0] is in userAtomsChoice list of elements
                          great, its one of the atoms we want

                          now do the same
                              if split_line[0] == "C":
                              # print("C atom found")
                              carbon_instance = elements.Carbon(atomnum=atom_number)
                              carbon_instance.cartesian = [split_line[1], split_line[2], split_line[3]]
                              carbon_list.append(carbon_instance)
                              listofatoms.append(carbon_instance)
                              atom_number = atom_number + 1

                  if it doesnt trigger the above if statement
                      atom_number = atom_number + 1


                  '''
            atoms_list = atomsChoice
            # print(atoms_list)
            atom_number = 1
            for line in coordinates_structure:
                print(line)
                if atom_number not in atoms_list:
                    # print("True")
                    # print(atom_number)
                    atom_number = atom_number + 1
                    continue
                else:
                    split_line = line.split()
                    if split_line[0] == "C":
                        # print("C atom found")
                        carbon_instance = elements.Carbon(atomnum=atom_number)
                        carbon_instance.cartesian = [split_line[1], split_line[2], split_line[3]]
                        carbon_list.append(carbon_instance)
                        listofatoms.append(carbon_instance)
                        atom_number = atom_number + 1

                    elif split_line[0] == "H":
                        # print("H atom found")
                        hydrogen_instance = elements.Hydrogen(atomnum=atom_number)
                        hydrogen_instance.cartesian = [split_line[1], split_line[2], split_line[3]]
                        hydrogen_list.append(hydrogen_instance)
                        listofatoms.append(hydrogen_instance)
                        atom_number = atom_number + 1

                    elif split_line[0] == "O":
                        # print("O atom found")
                        oxygen_instance = elements.Oxygen(atomnum=atom_number)
                        oxygen_instance.cartesian = [split_line[1], split_line[2], split_line[3]]
                        oxygen_list.append(oxygen_instance)
                        listofatoms.append(oxygen_instance)
                        atom_number = atom_number + 1

                    elif split_line[0] == "N":
                        # print("N atom found")
                        nitrogen_instance = elements.Nitrogen(atomnum=atom_number)
                        nitrogen_instance.cartesian = [split_line[1], split_line[2], split_line[3]]
                        nitrogen_list.append(nitrogen_instance)
                        listofatoms.append(nitrogen_instance)
                        atom_number = atom_number + 1

                    elif split_line[0] == "F":
                        # print("N atom found")
                        fluorine_instance = elements.Fluorine(atomnum=atom_number)
                        fluorine_instance.cartesian = [split_line[1], split_line[2], split_line[3]]
                        fluorine_list.append(fluorine_instance)
                        listofatoms.append(fluorine_instance)
                        atom_number = atom_number + 1

            print()
            print("Would you like to compute distances? Y/N")
            user_comp_distances = input("Distances (Y/N): ").upper()
            if user_comp_distances == "Y":
                print("Distances will be computed")
                comp_distances = True
                print()
                distances_indexes_list = []

                for atom_1 in atoms_list:

                    for atom_2 in atoms_list:
                        if atom_2 > atom_1:
                            distances_indexes_list.append([atom_1, atom_2])

                if not distances_indexes_list:
                    # this means that your criteria for atoms/elements creates no proper dihedrals
                    raise InvalidUserEntranceError("Error: Your criteria for atoms/elements creates no distances")

            elif user_comp_distances == "N":
                print("Distances will NOT be computed")
                comp_distances = False
                print()

            else:
                raise InvalidUserEntranceError("Error: Must Choose Y or N")

            print("Would you like to compute angles? Y/N")
            user_comp_angles = input("Angles (Y/N): ").upper()

            if user_comp_angles == "Y":
                print("Angles will be computed")
                comp_angles = True
                print()
                angles_indexes_list = []

                for atom_1 in atoms_list:

                    for atom_2 in atoms_list:
                        if atom_2 != atom_1:

                            for atom_3 in atoms_list:
                                if atom_3 not in [atom_1, atom_2] and atom_3 > atom_1:
                                    angles_indexes_list.append([atom_1,atom_2,atom_3])

                if not angles_indexes_list:
                    # this means that your criteria for atoms/elements creates no proper dihedrals
                    raise InvalidUserEntranceError("Error: Your criteria for atoms/elements creates no angles")

            elif user_comp_angles == "N":
                print("Angles will NOT be computed")
                comp_angles = False
                print()

            else:
                raise InvalidUserEntranceError("Error: Must Choose Y or N")

            print("Would you like to compute dihedrals? Y/N")
            user_comp_dihedrals = input("Dihedrals (Y/N): ").upper()

            if user_comp_dihedrals == "Y":
                print("Dihedrals will be computed")
                print()
                comp_dihedrals = True
                # dihedral_indexes_list = []
                #
                # for atom_1 in atoms_list:
                #
                #     for atom_2 in atoms_list:
                #         if atom_2 != atom_1:
                #
                #             for atom_3 in atoms_list:
                #                 if atom_3 > atom_2 and atom_3 != atom_1:
                #
                #                     for atom_4 in atoms_list:
                #
                #                         if atom_4 not in [atom_1, atom_2, atom_3] and atom_4 > atom_1:
                #                             dihedral_indexes_list.append([atom_1, atom_2, atom_3, atom_4])
                proper_dihedrals_atoms = []
                for atom1 in listofatoms:
                    for atom2 in listofatoms:
                        curr_distance = elements.dist(atom1, atom2)
                        if atom1.atom_num != atom2.atom_num and atom1.min_bd < curr_distance < atom1.max_bd and atom2.min_bd < curr_distance < atom2.max_bd:
                            # print(atom1.cartesian)
                            # print(atom2.cartesian)
                            # print(atom1.min_bd,atom1.max_bd)
                            # print(dist(atom1,atom2))
                            # print(atom2.min_bd,atom2.max_bd)
                            # sys.exit()

                            for atom3 in listofatoms:
                                if atom3.atom_num > atom2.atom_num and atom3.atom_num != atom1.atom_num and atom2.min_bd < elements.dist(
                                        atom2, atom3) < atom2.max_bd and atom3.min_bd < elements.dist(atom2,
                                                                                             atom3) < atom3.max_bd:
                                    # we keep going
                                    for atom4 in listofatoms:
                                        if atom4.atom_num > atom1.atom_num and atom4.atom_num not in [atom1.atom_num,
                                                                                                      atom2.atom_num,
                                                                                                      atom3.atom_num] and atom3.min_bd < elements.dist(
                                                atom3, atom4) < atom3.max_bd and atom4.min_bd < elements.dist(atom3,
                                                                                                     atom4) < atom4.max_bd:
                                            # print("True LOOP LAST")
                                            # print("Atom1: ", atom1.atom_num)
                                            # print("Atom2: ", atom2.atom_num)
                                            # print("Atom3: ", atom3.atom_num)
                                            # print("Atom4: ", atom4.atom_num)
                                            proper_dihedrals_atoms.append([atom1, atom2, atom3, atom4])

                                            """
                                            if [atom 2, atom 3] is in dihedral_indexes_list (but only position 1 and 2)
                                                                        [____, atom 2, atom 3, ___]
                                            if not any(isinstance([atom_4, atom_2, atom_3, atom_1], dihedral_indexes_list):
                                                test_equality = True
                                                
                                            if [atom 2, atom 3] is in dihedral_indexes_list[][1:2] 
                                            #
                                            # for dihedrals in dihedral_indexes_list:
                                            #     if atom_2 == dihedrals[1] and atom_3 == dihedrals[2] and atom_1 == dihedrals[3] and atom_4 == dihedrals[0]:
                                            #         test_equality = True
                                            #
                                            # if not test_equality:
                                            #     dihedral_indexes_list.append([atom_1, atom_2, atom_3, atom_4])
                                            """
                if not proper_dihedrals_atoms:
                    print(proper_dihedrals_atoms)
                    # this means that your criteria for atoms/elements creates no proper dihedrals
                    raise InvalidUserEntranceError(
                        "Error: Your criteria for atoms/elements creates no proper dihedrals")



            elif user_comp_dihedrals == "N":
                print("Dihedrals will NOT be computed")
                comp_dihedrals = False
                print()

            else:
                raise InvalidUserEntranceError("Error: Must Choose Y or N")

        except InvalidUserEntranceError as e:
            print()
            print(e.message)
            print("Restarting Questioning...")
            print()
            print("______________________________________________________")
            continue

        except ValueError:
            print()
            print("Value Error. You cannot put a word in list of atoms (integers)")
            print("Try Again...")
            print()
            print("______________________________________________________")
            continue
        break

    for file in xyz_files:
        # print(file)  # test works to open xyz file
        complete_path = user_directory + "/" + file  # need complete path to open file
        specific_file_object = open(complete_path)

        n_atoms = specific_file_object.readline()
        n_atoms = int(n_atoms)
        structure_name = specific_file_object.readline()  # subject to the operation performed prior to exporting XYZ; could change
        temp_xyz_df = specific_file_object.readlines()
        # print(temp_xyz_df)
        temp_xyz_contents_list = []
        temp_xyz_colname_list = []

        xyz_index_list.append(file)  # should be file name, will be used to change index from number to file name
        print("Files Found (updates):",xyz_index_list)
        atom_num = 1

        # XYZ
        #  Getting XYZ Content (Cartesian Coordinates)
        for elementxyzs in temp_xyz_df:
            # print(elementxyzs)
            split_elementxyzs = elementxyzs.split()
            if (len(split_elementxyzs) > 0):  # necessary because at the end of each xyz file, there are three blank lines
                # print(split_elementxyzs)
                # print(split_elementxyzs[0]) atom name/element name
                column_name = str(atom_num) + " " + split_elementxyzs[0]  # ex. 15 C; overall atom number and element

                temp_xyz_contents_list.append(split_elementxyzs[1])
                temp_xyz_colname_list.append(column_name + " -X")
                temp_xyz_contents_list.append(split_elementxyzs[2])
                temp_xyz_colname_list.append(column_name + " -Y")
                temp_xyz_contents_list.append(split_elementxyzs[3])
                temp_xyz_colname_list.append(column_name + " -Z")
                atom_num += 1
        master_xyz_contents_list.append(temp_xyz_contents_list)
        print("Cartesian Cordinates: COMPLETE")

        # DISTANCES
        # Getting Distances from each atom to all others
        if comp_distances:
            distances_calcs, distances_colmn = calc_distances(distances_indexes_list,temp_xyz_contents_list,
                                                                temp_xyz_colname_list)
        # ANGLES
        # Getting Angles from each atom to all others
        if comp_angles:
            angles_calcs, angles_colmn = calc_angles(angles_indexes_list,temp_xyz_contents_list,
                                                       temp_xyz_colname_list)
        # Dihedrals
        # Getting Dihedrals from each atom to all others
        if comp_dihedrals:
            # dihedral_calcs, dihedrals_colmn = calc_dihedrals(dihedral_indexes_list,temp_xyz_contents_list,
            #                                                    temp_xyz_colname_list)

            dihedral_calcs, dihedrals_colmn = calc_dihedrals_new(proper_dihedrals_atoms, temp_xyz_contents_list,
                                                             temp_xyz_colname_list)

    test_xyz_df = pd.DataFrame(master_xyz_contents_list,
                               columns=temp_xyz_colname_list)
    test_xyz_df.index = xyz_index_list

    if comp_distances:
        test_dist_df = pd.DataFrame(distances_calcs,
                                    columns=distances_colmn)
        test_dist_df.index = xyz_index_list

    if comp_angles:
        test_angle_df = pd.DataFrame(angles_calcs,
                                     columns=angles_colmn)
        test_angle_df.index = xyz_index_list

    if comp_dihedrals:
        test_dihedral_df = pd.DataFrame(dihedral_calcs,
                                     columns=dihedrals_colmn)
        test_dihedral_df.index = xyz_index_list

    if comp_distances and comp_angles and comp_dihedrals:
        frames = [test_dist_df, test_angle_df, test_dihedral_df]

    elif comp_distances and comp_angles:
        frames = [test_dist_df, test_angle_df]

    elif comp_distances and comp_dihedrals:
        frames = [test_dist_df, test_dihedral_df]

    elif comp_angles and comp_dihedrals:
        frames = [test_angle_df, test_dihedral_df]

    elif comp_distances:
        frames = [test_dist_df]

    elif comp_angles:
        frames = [test_angle_df]

    elif comp_dihedrals:
        frames = [test_dihedral_df]

    calculations_df = pd.concat(frames, axis=1)
    specific_file_object.close()
    return calculations_df
    # return test_xyz_df,test_dist_df,test_angle_df,test_dihedral_df,calculations_df



import os
import pandas as pd
import numpy as np
import sys

class Element:
    def __init__(self,atomnum,min_bond_dist=1,max_bond_dist=2.5,min_num_bonds=1,max_num_bonds=8,coordinates = []):
        self.atom_num = atomnum
        self.min_bd = min_bond_dist
        self.max_bd = max_bond_dist
        self.min_nb = min_num_bonds
        self.max_nb = max_num_bonds
        self.cartesian = coordinates


class Carbon(Element):
    def __init__(self,atomnum,min_bond_dist=1.01,max_bond_dist=2.13,min_num_bonds=4,max_num_bonds=4, coordinates = []):
        self.atom_num = atomnum
        self.min_bd = min_bond_dist
        self.max_bd = max_bond_dist
        self.min_nb = min_num_bonds
        self.max_nb = max_num_bonds
        self.cartesian = coordinates


class Oxygen(Element):
    def __init__(self,atomnum,min_bond_dist=0.90,max_bond_dist=1.94,min_num_bonds=2,max_num_bonds=2, coordinates = []):
        self.atom_num = atomnum
        self.min_bd = min_bond_dist
        self.max_bd = max_bond_dist
        self.min_nb = min_num_bonds
        self.max_nb = max_num_bonds
        self.cartesian = coordinates


class Hydrogen(Element):
    def __init__(self,atomnum,min_bond_dist=0.70,max_bond_dist=1.64,min_num_bonds=1,max_num_bonds=1, coordinates = []):
        self.atom_num = atomnum
        self.min_bd = min_bond_dist
        self.max_bd = max_bond_dist
        self.min_nb = min_num_bonds
        self.max_nb = max_num_bonds
        self.cartesian = coordinates


class Nitrogen(Element):
    def __init__(self,atomnum,min_bond_dist=0.90,max_bond_dist=1.80,min_num_bonds=1,max_num_bonds=4, coordinates = []):
        self.atom_num = atomnum
        self.min_bd = min_bond_dist
        self.max_bd = max_bond_dist
        self.min_nb = min_num_bonds
        self.max_nb = max_num_bonds
        self.cartesian = coordinates


class Fluorine(Element):
    def __init__(self,atomnum,min_bond_dist=0.90,max_bond_dist=1.54,min_num_bonds=1,max_num_bonds=4, coordinates = []):
        self.atom_num = atomnum
        self.min_bd = min_bond_dist
        self.max_bd = max_bond_dist
        self.min_nb = min_num_bonds
        self.max_nb = max_num_bonds
        self.cartesian = coordinates

# c1 = Carbon()
# print(c1.max_bd)
#
# o1 = Oxygen()
# print(o1.max_bd)

########################################################################################################################
#  Testing How Elements would work

# calc_distance = True
# calc_dihedrals = True
# user_input_directory = "/Users/matthewnwerem/Chapman University/OMO Research Group - Project Conf. Analysis - Nwerem - proj_conf_analysis/Code Versions/Conformational_Analysis/xyz_file_test"
# user_input_directory = "/Users/matthewnwerem/Chapman University/OMO Research Group - Project Conf. Analysis - Nwerem - proj_conf_analysis/Code Versions/Conformational_Analysis/pentane_file_test"
# user_input_directory = "/Users/matthewnwerem/Chapman University/OMO Research Group - Project Conf. Analysis - Nwerem - proj_conf_analysis/Code Versions/Conformational_Analysis/cycloheptadecane_file_test"
#
# directory_files = os.listdir(user_input_directory)
# xyz_files = []
# xyz_index_list = []
# master_xyz_contents_list = []
# distances_indexes_list = []
# dihedral_indexes_list = []
# master_dist_contents_list = []
# calculations_list = []
#
#
# for file in directory_files:
#     if file.endswith(".xyz"):
#         xyz_files.append(file)
#
# complete_path = user_input_directory + "/" + xyz_files[0]
# print(complete_path)
# specific_file_object = open(complete_path)
# n_atoms = specific_file_object.readline()
# structure_name = specific_file_object.readline()
# coordinates_structure = specific_file_object.readlines()
# carbon_list = []
# hydrogen_list = []
# oxygen_list = []
# listofatoms = []
#
# atom_number = 1
# for line in coordinates_structure:
#     split_line = line.split()
#     if split_line[0] == "C":
#         print("C atom found")
#         carbon_instance = Carbon(atomnum=atom_number)
#         carbon_instance.cartesian = [split_line[1],split_line[2],split_line[3]]
#         carbon_list.append(carbon_instance)
#         listofatoms.append(carbon_instance)
#         atom_number = atom_number + 1
#
#     elif split_line[0] == "H":
#         print("H atom found")
#         hydrogen_instance = Hydrogen(atomnum=atom_number)
#         hydrogen_instance.cartesian = [split_line[1],split_line[2],split_line[3]]
#         hydrogen_list.append(hydrogen_instance)
#         listofatoms.append(hydrogen_instance)
#         atom_number = atom_number + 1
#
#     elif split_line[0] == "O":
#         print("O atom found")
#         oxygen_instance = Hydrogen(atomnum=atom_number)
#         oxygen_instance.cartesian = [split_line[1],split_line[2],split_line[3]]
#         oxygen_list.append(oxygen_instance)
#         listofatoms.append(oxygen_instance)
#         atom_number = atom_number + 1
#
#     else:
#         print(split_line)
#         atom_number = atom_number + 1
#
#
# print("C LIST length")
# print(len(carbon_list))
# print("C****************C")
# print()
#
# print("H LIST length")
# print(len(hydrogen_list))
# print("H****************H")
# print()
#
# print("O LIST length")
# print(len(oxygen_list))
# print("O****************O")
# print()

# for hydrogens in hydrogen_list:
#     print(hydrogens.cartesian)


def dist(atom1,atom2):
    a1 = atom1.cartesian
    a1_array = np.array(list(map(float,a1)))
    a2 = atom2.cartesian
    a2_array = np.array(list(map(float,a2)))
    v1 = a2_array - a1_array
    dist = np.linalg.norm(v1)
    return dist

#
# proper_dihedrals_atoms = []
# for atom1 in listofatoms:
#     for atom2 in listofatoms:
#         curr_distance = dist(atom1, atom2)
#         if atom1.atom_num != atom2.atom_num and atom1.min_bd < curr_distance < atom1.max_bd and atom2.min_bd < curr_distance <  atom2.max_bd:
#             # print(atom1.cartesian)
#             # print(atom2.cartesian)
#             # print(atom1.min_bd,atom1.max_bd)
#             # print(dist(atom1,atom2))
#             # print(atom2.min_bd,atom2.max_bd)
#             # sys.exit()
#
#             for atom3 in listofatoms:
#                 if atom3.atom_num > atom2.atom_num and atom3.atom_num != atom1.atom_num and atom2.min_bd < dist(atom2,atom3) < atom2.max_bd and atom3.min_bd < dist(atom2,atom3) <  atom3.max_bd:
#                     #we keep going
#                     for atom4 in listofatoms:
#                         if atom4.atom_num > atom1.atom_num and atom4.atom_num not in [atom1.atom_num,atom2.atom_num,atom3.atom_num] and atom3.min_bd < dist(atom3,atom4) < atom3.max_bd and atom4.min_bd < dist(atom3,atom4) < atom4.max_bd:
#                             print("True LOOP LAST")
#                             print("Atom1: ", atom1.atom_num)
#                             print("Atom2: ", atom2.atom_num)
#                             print("Atom3: ", atom3.atom_num)
#                             print("Atom4: ", atom4.atom_num)
#                             proper_dihedrals_atoms.append([atom1,atom2,atom3,atom4])
#
# # for items in proper_dihedrals_atoms:
# #     print(items[0].cartesian)
# #     print(items[1].cartesian)
# #     print(items[2].cartesian)
# #     print(items[3].cartesian)
# #     print()
# #     print()
# #     sys.exit()
#
# print(len(listofatoms))
# print(len(proper_dihedrals_atoms))
#
# for elements in proper_dihedrals_atoms:
#     for atoms in elements:
#         print(atoms.atom_num)
#
#     print()


# specific_file_object = open(complete_path)
# n_atoms_distangdih = specific_file_object.readline()
# n_atoms_distangdih = int(n_atoms_distangdih)
#
# atoms_list = list(range(1, n_atoms_distangdih + 1))
#
# print("Which atoms you would like to compute calculations on.")
# print("All you need to type is the atom NUMBER, no spaces (ex. 1,5,7,8,10)")
# print("If you would like all, enter 'all'")
#
# # userAtomsChoice = input("Atoms: ")
# atomsChoice = list(range(1, n_atoms_distangdih + 1)) #doing "all" for the first couple runs
#
# # if userAtomsChoice == "all":
# #     atomsChoice = list(range(1, n_atoms_distangdih + 1))
#
# # else:
# #     #  remove duplicates using set()
# #     split_Choices = userAtomsChoice.split(",")
# #
# #     # removing any possible empty strings
# #     # https://stackoverflow.com/questions/3845423/remove-empty-strings-from-a-list-of-strings
# #     split_Choices = list(filter(None, split_Choices))
# #
# #     atomsChoice = list(map(int, set(split_Choices)))
# #     atomsChoice.sort()
# #     #  list(map(int, test_list) will make each number in list an int
#
# atoms_list = atomsChoice
#
# print("Compute distances? Yes")
# for atom_1 in atoms_list:
#
#     for atom_2 in atoms_list:
#         if atom_2 > atom_1:
#             distances_indexes_list.append([atom_1, atom_2])
#
#
# # print("Would you like to compute angles? Y/N")
# # user_comp_angles = input("Angles (Y/N): ").upper()
# #
# # if user_comp_angles == "Y":
# #     print("Angles will be computed")
# #     comp_angles = True
# #     print()
# #     angles_indexes_list = []
# #
# #     for atom_1 in atoms_list:
# #
# #         for atom_2 in atoms_list:
# #             if atom_2 != atom_1:
# #
# #                 for atom_3 in atoms_list:
# #                     if atom_3 not in [atom_1, atom_2] and atom_3 > atom_1:
# #                         angles_indexes_list.append([atom_1, atom_2, atom_3])
# #
#
#
# print("Compute dihedrals? Yes")
# for atom_1 in atoms_list:
#
#     for atom_2 in atoms_list:
#         if atom_2 != atom_1:
#
#             for atom_3 in atoms_list:
#                 if atom_3 > atom_2 and atom_3 != atom_1:
#
#                     for atom_4 in atoms_list:
#
#                         if atom_4 not in [atom_1, atom_2, atom_3] and atom_4 > atom_1:
#                             dihedral_indexes_list.append([atom_1, atom_2, atom_3, atom_4])
#
#
# for file in xyz_files:
#     # print(file)  # test works to open xyz file
#     complete_path = user_input_directory + "/" + file  # need complete path to open file
#     specific_file_object = open(complete_path)
#
#     n_atoms = specific_file_object.readline()
#     n_atoms = int(n_atoms)
#     structure_name = specific_file_object.readline()  # subject to the operation performed prior to exporting XYZ; could change
#     temp_xyz_df = specific_file_object.readlines()
#     # print(temp_xyz_df)
#     temp_xyz_contents_list = []
#     temp_xyz_colname_list = []
#
#     xyz_index_list.append(file)  # should be file name, will be used to change index from number to file name
#     print("Files Found (updates):", xyz_index_list)
#     atom_num = 1
#
#     for elementxyzs in temp_xyz_df:
#         # print(elementxyzs)
#         split_elementxyzs = elementxyzs.split()
#         if (len(split_elementxyzs) > 0):  # necessary because at the end of each xyz file, there are three blank lines
#             # print(split_elementxyzs)
#             # print(split_elementxyzs[0]) atom name/element name
#             column_name = str(atom_num) + " " + split_elementxyzs[0]  # ex. 15 C; overall atom number and element
#
#             temp_xyz_contents_list.append(split_elementxyzs[1])
#             temp_xyz_colname_list.append(column_name + " -X")
#             temp_xyz_contents_list.append(split_elementxyzs[2])
#             temp_xyz_colname_list.append(column_name + " -Y")
#             temp_xyz_contents_list.append(split_elementxyzs[3])
#             temp_xyz_colname_list.append(column_name + " -Z")
#             atom_num += 1
#     master_xyz_contents_list.append(temp_xyz_contents_list)
#     print("Cartesian Cordinates: COMPLETE")
#
# test_xyz_df = pd.DataFrame(master_xyz_contents_list,
#                                columns=temp_xyz_colname_list)
# test_xyz_df.index = xyz_index_list
#
# # print(test_xyz_df)
#
# ## so we have the xyz's correctly done cool
#
# temp_dist_list = []
# dist_colname_list = []
#
# for dist_list in distances_indexes_list:
#     a1 = [float(temp_xyz_contents_list[3 * dist_list[0] - 3]),
#           float(temp_xyz_contents_list[3 * dist_list[0] - 2]),
#           float(temp_xyz_contents_list[3 * dist_list[0] - 1])]
#     a1_array = np.array(a1)
#
#     a2 = [float(temp_xyz_contents_list[3 * dist_list[1] - 3]),
#           float(temp_xyz_contents_list[3 * dist_list[1] - 2]),
#           float(temp_xyz_contents_list[3 * dist_list[1] - 1])]
#     a2_array = np.array(a2)
#
#     v1 = a2_array - a1_array
#     dist = np.linalg.norm(v1)
#     temp_dist_list.append(dist)
#     dist_colname_list.append(temp_xyz_colname_list[3 * dist_list[0] - 1][:-2] +
#                              "to " + temp_xyz_colname_list[3 * dist_list[1] - 1][:-2])  # Ex. 15 C to 18 H
#
# master_dist_contents_list.append(temp_dist_list)
# calculations_list.append(temp_dist_list)
# print("Distances: COMPLETE")
#
# test_dist_df = pd.DataFrame(master_dist_contents_list, columns=dist_colname_list)
# # test_dist_df.index = xyz_index_list
# print(test_dist_df)
# proper_dihedrals_indices = []
# #
# #I need to know which atom is which, this means when i read the file, I have to create an instance of the element
#
#
# # for indx,atom1 in enumerate(dihedral_indexes_list):
# #
# #     for indx2,atom2 in enumerate(dihedral_indexes_list):
# #         # if atom2 != atom1 and dist(atom1, atom2) > 0.8 and dist(atom1, atom2) < 1.7
# #         if atom2!= atom1 and calculations_list[indx+indx2] > 0.8
# # #             # we keep going
# # #
# # #             for atom3 in dihedral_indexes_list:
# #                 if atom2 not in [atom1, atom2] and dist(atom2, atom3) > 0.8
# #                     and dist(atom2, atom3) < 1.7:
# #                 # we keep going
# #
# #                 for atom4 in dihedral_indexes_list:
# #                     if atom4 not in [atom1, atom2, atom3] and dist(atom4, atom4) > 0.8
# #                         and dist(atom4, atom4) < 1.7:
# #
# #                 # this means if it gets this far, we have unique atoms that are
# #                 # "bonded" contiguously sorta
# #                 proper_dihedrals_indices.append([atom1, atom2, atom3, atom4])


from setuptools import setup
from codecs import open as copen
from os import path

here = path.abspath(path.dirname(__file__))

# with open("README.md", "r") as fh:
#     long_description = fh.read()


# Get the long description from the README file
with copen(path.join(here, "README.md"), encoding="utf-8") as f:
    long_description = f.read()

# Get the dependencies and installs
with copen(path.join(here, "requirements.txt"), encoding="utf-8") as f:
    all_reqs = f.read().split("\n")

install_requires = [x.strip() for x in all_reqs if "git+" not in x]
dependency_links = [x.strip().replace("git+", "") for x in all_reqs if x.startswith("git+")]

setup(
    name= "omolab_conf_analysis",
    version="0.1.10",
    description="Uses PCA and K-means clustering to separate conformational ensembles. (xyz-files only)",
    py_modules=["clustering","ConformationalAnalysis","pymol","xyz",
                "pca_creation","elements","mGenerate_conf_analysis","mGenerate_PyMOL_Session"],
    scripts=["mGenerate_conf_analysis.sh","mGenerate_PyMOL_Session.sh"],
    include_package_data=True,
    classifiers = [
        "Programming Language :: Python :: 3",
        "Development Status :: 4 - Beta",
        "Environment :: MacOS X",
        "Framework :: Matplotlib",
        "Intended Audience :: Education",
        "Intended Audience :: Science/Research",
        "License :: Free For Educational Use",
        "License :: Free For Home Use",
        "License :: Free for non-commercial use",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Operating System :: MacOS",
        "Operating System :: Microsoft :: Windows",
        "Topic :: Scientific/Engineering :: Chemistry",
        "Topic :: Scientific/Engineering :: Information Analysis"],
    long_description=long_description,
    long_description_content_type="text/markdown",
    install_requires= install_requires,
    dependency_links= dependency_links,
    url = "https://github.com/mattnw1/Conformational_Analysis",
    author="Matthew Nwerem",
    author_email="nwere100@mail.chapman.edu"

)

#mGenerate_PyMOL_Session.sh cannot be included here because it is not a .py file, must find another way to add it
